@@ -148,15 +148,15 @@ def main():
 		CNetObjHandler();
 
 		int ValidateObj(int Type, void *pData, int Size);
-		const char *GetObjName(int Type);
-		int GetObjSize(int Type);
-		int NumObjCorrections();
-		const char *CorrectedObjOn();
+		const char *GetObjName(int Type) const;
+		int GetObjSize(int Type) const;
+		int NumObjCorrections() const;
+		const char *CorrectedObjOn() const;
 
-		const char *GetMsgName(int Type);
+		const char *GetMsgName(int Type) const;
 		void *SecureUnpackMsg(int Type, CUnpacker *pUnpacker);
 		bool TeeHistorianRecordMsg(int Type);
-		const char *FailedMsgOn();
+		const char *FailedMsgOn() const;
 	};
 
 	""")
@@ -180,9 +180,9 @@ def main():
 		lines += ['\tm_NumObjCorrections = 0;']
 		lines += ['}']
 		lines += ['']
-		lines += ['int CNetObjHandler::NumObjCorrections() { return m_NumObjCorrections; }']
-		lines += ['const char *CNetObjHandler::CorrectedObjOn() { return m_pObjCorrectedOn; }']
-		lines += ['const char *CNetObjHandler::FailedMsgOn() { return m_pMsgFailedOn; }']
+		lines += ['int CNetObjHandler::NumObjCorrections() const { return m_NumObjCorrections; }']
+		lines += ['const char *CNetObjHandler::CorrectedObjOn() const { return m_pObjCorrectedOn; }']
+		lines += ['const char *CNetObjHandler::FailedMsgOn() const { return m_pMsgFailedOn; }']
 		lines += ['']
 		lines += ['']
 		lines += ['']
@@ -219,14 +219,14 @@ def main():
 		lines += ['};']
 		lines += ['']
 
-		lines += ['const char *CNetObjHandler::GetObjName(int Type)']
+		lines += ['const char *CNetObjHandler::GetObjName(int Type) const']
 		lines += ['{']
 		lines += ['\tif(Type < 0 || Type >= NUM_NETOBJTYPES) return "(out of range)";']
 		lines += ['\treturn ms_apObjNames[Type];']
 		lines += ['}']
 		lines += ['']
 
-		lines += ['int CNetObjHandler::GetObjSize(int Type)']
+		lines += ['int CNetObjHandler::GetObjSize(int Type) const']
 		lines += ['{']
 		lines += ['\tif(Type < 0 || Type >= NUM_NETOBJTYPES) return 0;']
 		lines += ['\treturn ms_aObjSizes[Type];']
@@ -234,7 +234,7 @@ def main():
 		lines += ['']
 
 
-		lines += ['const char *CNetObjHandler::GetMsgName(int Type)']
+		lines += ['const char *CNetObjHandler::GetMsgName(int Type) const']
 		lines += ['{']
 		lines += ['\tif(Type < 0 || Type >= NUM_NETMSGTYPES) return "(out of range)";']
 		lines += ['\treturn ms_apMsgNames[Type];']
