@@ -238,6 +238,7 @@ image_guiicons = Image("guiicons", "gui_icons.png")
 image_arrow = Image("arrow", "arrow.png")
 image_demobuttons2 = Image("demobuttons2", "demo_buttons2.png")
 image_audio_source = Image("audio_source", "editor/audio_source.png")
+image_strongweak = Image("strongweak", "strong_weak.png")
 
 container.images.Add(image_null)
 container.images.Add(image_game)
@@ -256,6 +257,7 @@ container.images.Add(image_guiicons)
 container.images.Add(image_arrow)
 container.images.Add(image_demobuttons2)
 container.images.Add(image_audio_source)
+container.images.Add(image_strongweak)
 
 container.pickups.Add(Pickup("health"))
 container.pickups.Add(Pickup("armor"))
@@ -274,6 +276,7 @@ set_guibuttons = SpriteSet("guibuttons", image_guibuttons, 12, 4)
 set_guiicons = SpriteSet("guiicons", image_guiicons, 12, 2)
 set_demobuttons2 = SpriteSet("demobuttons2", image_demobuttons2, 4, 1)
 set_audio_source = SpriteSet("audio_source", image_audio_source, 1, 1)
+set_strongweak = SpriteSet("strongweak", image_strongweak, 2, 1)
 
 container.spritesets.Add(set_particles)
 container.spritesets.Add(set_game)
@@ -287,6 +290,7 @@ container.spritesets.Add(set_guibuttons)
 container.spritesets.Add(set_guiicons)
 container.spritesets.Add(set_demobuttons2)
 container.spritesets.Add(set_audio_source)
+container.spritesets.Add(set_strongweak)
 
 container.sprites.Add(Sprite("part_slice", set_particles, 0,0,1,1))
 container.sprites.Add(Sprite("part_ball", set_particles, 1,0,1,1))
@@ -429,6 +433,9 @@ container.sprites.Add(Sprite("guiicon_friend", set_guiicons, 8,0,4,2))
 
 container.sprites.Add(Sprite("audio_source", set_audio_source, 0,0,1,1))
 
+container.sprites.Add(Sprite("hook_strong", set_strongweak, 0,0,1,1))
+container.sprites.Add(Sprite("hook_weak", set_strongweak, 1,0,1,1))
+
 anim = Animation("base")
 anim.body.frames.Add(AnimKeyframe(0, 0, -4, 0))
 anim.back_foot.frames.Add(AnimKeyframe(0, 0, 10, 0))
