@@ -1,4 +1,6 @@
-#include "antibot_data.h"
+#define ANTIBOTAPI DYNAMIC_EXPORT
+
+#include "antibot_interface.h"
 
 static CAntibotData *g_pData;
 
@@ -26,12 +28,14 @@ void AntibotOnPlayerDestroy(int /*ClientID*/) {}
 void AntibotOnSpawn(int /*ClientID*/) {}
 void AntibotOnHammerFireReloading(int /*ClientID*/) {}
 void AntibotOnHammerFire(int /*ClientID*/) {}
-void AntibotOnHammerHit(int /*ClientID*/) {}
+void AntibotOnHammerHit(int /*ClientID*/, int /*TargetID*/) {}
 void AntibotOnDirectInput(int /*ClientID*/) {}
 void AntibotOnCharacterTick(int /*ClientID*/) {}
 void AntibotOnHookAttach(int /*ClientID*/, bool /*Player*/) {}
 void AntibotOnEngineTick(void) {}
 void AntibotOnEngineClientJoin(int /*ClientID*/, bool /*Sixup*/) {}
 void AntibotOnEngineClientDrop(int /*ClientID*/, const char * /*pReason*/) {}
-void AntibotOnEngineClientMessage(int /*ClientID*/, const void * /*pData*/, int /*Size*/, int /*Flags*/) {}
+bool AntibotOnEngineClientMessage(int /*ClientID*/, const void * /*pData*/, int /*Size*/, int /*Flags*/) { return false; }
+bool AntibotOnEngineServerMessage(int /*ClientID*/, const void * /*pData*/, int /*Size*/, int /*Flags*/) { return false; }
+bool AntibotOnEngineSimulateClientMessage(int * /*pClientID*/, void * /*pBuffer*/, int /*BufferSize*/, int * /*pOutSize*/, int * /*pFlags*/) { return false; }
 }
