@@ -12,16 +12,12 @@
 #include <engine/external/md5/md5.h>
 #endif
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #if defined(CONF_OPENSSL)
 // SHA256_CTX is defined in <openssl/sha.h>
 #else
 typedef struct
 {
-	uint64 length;
+	uint64_t length;
 	uint32_t state[8];
 	uint32_t curlen;
 	unsigned char buf[64];
@@ -37,8 +33,4 @@ void md5_init(MD5_CTX *ctxt);
 void md5_update(MD5_CTX *ctxt, const void *data, size_t data_len);
 MD5_DIGEST md5_finish(MD5_CTX *ctxt);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif // BASE_HASH_CTXT_H
