@@ -15,6 +15,7 @@
 #endif
 
 #include <stddef.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <time.h>
 
@@ -27,7 +28,7 @@
 #include <sys/socket.h>
 #endif
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 extern "C" {
 #endif
 
@@ -41,16 +42,12 @@ extern "C" {
 		msg - Message that should be printed if the test fails.
 
 	Remarks:
-		Does nothing in release version
+		Also works in release mode.
 
 	See Also:
 		<dbg_break>
 */
-#ifdef CONF_DEBUG
 #define dbg_assert(test, msg) dbg_assert_imp(__FILE__, __LINE__, test, msg)
-#else
-#define dbg_assert(test, msg)
-#endif
 void dbg_assert_imp(const char *filename, int line, int test, const char *msg);
 
 #ifdef __clang_analyzer__
@@ -70,17 +67,12 @@ void dbg_assert_imp(const char *filename, int line, int test, const char *msg);
 		Breaks into the debugger.
 
 	Remarks:
-		Does nothing in release version
+		Also works in release mode.
 
 	See Also:
 		<dbg_assert>
 */
-#ifdef CONF_DEBUG
-#define dbg_break() dbg_break_imp()
-#else
-#define dbg_break()
-#endif
-void dbg_break_imp(void);
+void dbg_break();
 
 /*
 	Function: dbg_msg
@@ -92,7 +84,7 @@ void dbg_break_imp(void);
 		fmt - A printf styled format string.
 
 	Remarks:
-		Also works in release version
+		Also works in release mode.
 
 	See Also:
 		<dbg_assert>
@@ -169,12 +161,14 @@ enum
 {
 	IOFLAG_READ = 1,
 	IOFLAG_WRITE = 2,
-	IOFLAG_RANDOM = 4,
-	IOFLAG_APPEND = 8,
+	IOFLAG_APPEND = 4,
+	IOFLAG_SKIP_BOM = 8,
 
 	IOSEEK_START = 0,
 	IOSEEK_CUR = 1,
-	IOSEEK_END = 2
+	IOSEEK_END = 2,
+
+	IO_MAX_PATH_LENGTH = 512,
 };
 
 typedef struct IOINTERNAL *IOHANDLE;
@@ -185,7 +179,7 @@ typedef struct IOINTERNAL *IOHANDLE;
 
 	Parameters:
 		filename - File to open.
-		flags - A set of flags. IOFLAG_READ, IOFLAG_WRITE, IOFLAG_RANDOM, IOFLAG_APPEND.
+		flags - A set of flags. IOFLAG_READ, IOFLAG_WRITE, IOFLAG_APPEND, IOFLAG_SKIP_BOM.
 
 	Returns:
 		Returns a handle to the file on success and 0 on failure.
@@ -309,6 +303,18 @@ int io_close(IOHANDLE io);
 */
 int io_flush(IOHANDLE io);
 
+/*
+	Function: io_sync
+		Synchronize file changes to disk.
+
+	Parameters:
+		io - Handle to the file.
+
+	Returns:
+		Returns 0 on success.
+*/
+int io_sync(IOHANDLE io);
+
 /*
 	Function: io_error
 		Checks whether an error occurred during I/O with the file.
@@ -325,19 +331,25 @@ int io_error(IOHANDLE io);
 	Function: io_stdin
 		Returns an <IOHANDLE> to the standard input.
 */
-IOHANDLE io_stdin(void);
+IOHANDLE io_stdin();
 
 /*
 	Function: io_stdout
 		Returns an <IOHANDLE> to the standard output.
 */
-IOHANDLE io_stdout(void);
+IOHANDLE io_stdout();
 
 /*
 	Function: io_stderr
 		Returns an <IOHANDLE> to the standard error.
 */
-IOHANDLE io_stderr(void);
+IOHANDLE io_stderr();
+
+/*
+	Function: io_current_exe
+		Returns an <IOHANDLE> to the current executable.
+*/
+IOHANDLE io_current_exe();
 
 typedef struct ASYNCIO ASYNCIO;
 
@@ -503,7 +515,7 @@ void thread_wait(void *thread);
 	Function: thread_yield
 		Yield the current threads execution slice.
 */
-void thread_yield(void);
+void thread_yield();
 
 /*
 	Function: thread_detach
@@ -601,7 +613,7 @@ void *thread_init_and_detach(void (*threadfunc)(void *), void *user, const char
 /* Group: Locks */
 typedef CAPABILITY("mutex") void *LOCK;
 
-LOCK lock_create(void);
+LOCK lock_create();
 void lock_destroy(LOCK lock);
 
 int lock_trylock(LOCK lock) TRY_ACQUIRE(1, lock);
@@ -626,23 +638,7 @@ void sphore_wait(SEMAPHORE *sem);
 void sphore_signal(SEMAPHORE *sem);
 void sphore_destroy(SEMAPHORE *sem);
 
-/* Group: Timer */
-#ifdef __GNUC__
-/* if compiled with -pedantic-errors it will complain about long
-	not being a C90 thing.
-*/
-#ifdef CONF_PLATFORM_HAIKU
-#include <SupportDefs.h>
-#else
-__extension__ typedef long long int64;
-#endif
-__extension__ typedef unsigned long long uint64;
-#else
-typedef long long int64;
-typedef unsigned long long uint64;
-#endif
-
-void set_new_tick(void);
+void set_new_tick();
 
 /*
 	Function: time_get_impl
@@ -654,7 +650,7 @@ void set_new_tick(void);
 	Remarks:
 		To know how fast the timer is ticking, see <time_freq>.
 */
-int64 time_get_impl(void);
+int64_t time_get_impl();
 
 /*
 	Function: time_get
@@ -667,7 +663,7 @@ int64 time_get_impl(void);
 		To know how fast the timer is ticking, see <time_freq>.
 		Uses <time_get_impl> to fetch the sample.
 */
-int64 time_get(void);
+int64_t time_get();
 
 /*
 	Function: time_freq
@@ -676,7 +672,7 @@ int64 time_get(void);
 	Returns:
 		Returns the frequency of the high resolution timer.
 */
-int64 time_freq(void);
+int64_t time_freq();
 
 /*
 	Function: time_timestamp
@@ -685,7 +681,7 @@ int64 time_freq(void);
 	Returns:
 		The time as a UNIX timestamp
 */
-int time_timestamp(void);
+int time_timestamp();
 
 /*
 	Function: time_houroftheday
@@ -694,7 +690,7 @@ int time_timestamp(void);
 	Returns:
 		The current hour of the day
 */
-int time_houroftheday(void);
+int time_houroftheday();
 
 enum
 {
@@ -712,7 +708,7 @@ enum
 	Returns:
 		one of the SEASON_* enum literals
 */
-int time_season(void);
+int time_season();
 
 /*
 Function: time_get_microseconds
@@ -721,7 +717,7 @@ Fetches a sample from a high resolution timer and converts it in microseconds.
 Returns:
 Current value of the timer in microseconds.
 */
-int64 time_get_microseconds(void);
+int64_t time_get_microseconds();
 
 /* Group: Network General */
 typedef struct
@@ -766,7 +762,7 @@ typedef struct sockaddr_un UNIXSOCKETADDR;
 		You must call this function before using any other network
 		functions.
 */
-int net_init(void);
+int net_init();
 
 /*
 	Function: net_host_lookup
@@ -1022,7 +1018,7 @@ int net_tcp_close(NETSOCKET sock);
 	Returns:
 		On success it returns a handle to the socket. On failure it returns -1.
 */
-UNIXSOCKET net_unix_create_unnamed(void);
+UNIXSOCKET net_unix_create_unnamed();
 
 /*
 	Function: net_unix_send
@@ -1587,7 +1583,7 @@ enum
 		- Guarantees that buffer string will contain zero-termination, assuming
 		  buffer_size > 0.
 */
-int str_time(int64 centisecs, int format, char *buffer, int buffer_size);
+int str_time(int64_t centisecs, int format, char *buffer, int buffer_size);
 int str_time_float(float secs, int format, char *buffer, int buffer_size);
 
 /*
@@ -1613,14 +1609,29 @@ void str_escape(char **dst, const char *src, const char *end);
 		cb - Callback function to call for each entry
 		type - Type of the directory
 		user - Pointer to give to the callback
-
-	Returns:
-		Always returns 0.
 */
 typedef int (*FS_LISTDIR_CALLBACK)(const char *name, int is_dir, int dir_type, void *user);
-typedef int (*FS_LISTDIR_INFO_CALLBACK)(const char *name, time_t date, int is_dir, int dir_type, void *user);
-int fs_listdir(const char *dir, FS_LISTDIR_CALLBACK cb, int type, void *user);
-int fs_listdir_info(const char *dir, FS_LISTDIR_INFO_CALLBACK cb, int type, void *user);
+void fs_listdir(const char *dir, FS_LISTDIR_CALLBACK cb, int type, void *user);
+
+typedef struct
+{
+	const char *m_pName;
+	time_t m_TimeCreated; // seconds since UNIX Epoch
+	time_t m_TimeModified; // seconds since UNIX Epoch
+} CFsFileInfo;
+
+/*
+	Function: fs_listdir_fileinfo
+		Lists the files in a directory and gets additional file information
+
+	Parameters:
+		dir - Directory to list
+		cb - Callback function to call for each entry
+		type - Type of the directory
+		user - Pointer to give to the callback
+*/
+typedef int (*FS_LISTDIR_CALLBACK_FILEINFO)(const CFsFileInfo *info, int is_dir, int dir_type, void *user);
+void fs_listdir_fileinfo(const char *dir, FS_LISTDIR_CALLBACK_FILEINFO cb, int type, void *user);
 
 /*
 	Function: fs_makedir
@@ -1688,12 +1699,6 @@ int fs_storage_path(const char *appname, char *path, int max);
 */
 int fs_is_dir(const char *path);
 
-/*
-	Function: fs_getmtime
-		Gets the modification time of a file
-*/
-time_t fs_getmtime(const char *path);
-
 /*
 	Function: fs_chdir
 		Changes current working directory
@@ -1759,6 +1764,23 @@ int fs_remove(const char *filename);
 */
 int fs_rename(const char *oldname, const char *newname);
 
+/*
+	Function: fs_file_time
+		Gets the creation and the last modification date of a file.
+
+	Parameters:
+		name - The filename.
+		created - Pointer to time_t
+		modified - Pointer to time_t
+
+	Returns:
+		0 on success non-zero on failure
+
+	Remarks:
+		- Returned time is in seconds since UNIX Epoch
+*/
+int fs_file_time(const char *name, time_t *created, time_t *modified);
+
 /*
 	Group: Undocumented
 */
@@ -1789,48 +1811,63 @@ int net_set_blocking(NETSOCKET sock);
 
 	DOCTODO: serp
 */
-int net_errno(void);
+int net_errno();
 
 /*
 	Function: net_would_block
 
 	DOCTODO: serp
 */
-int net_would_block(void);
+int net_would_block();
 
 int net_socket_read_wait(NETSOCKET sock, int time);
 
 /*
 	Function: open_link
 		Opens a link in the browser.
-	
+
 	Parameters:
 		link - The link to open in a browser.
-	
+
 	Returns:
 		Returns 1 on success, 0 on failure.
-	
+
 	Remarks:
-		This may not be called with untrusted input or it'll result in arbitrary code execution.
+		This may not be called with untrusted input or it'll result in arbitrary code execution, especially on Windows.
 */
 int open_link(const char *link);
 
+/*
+	Function: open_file
+		Opens a file or directory with default program.
+
+	Parameters:
+		path - The path to open.
+
+	Returns:
+		Returns 1 on success, 0 on failure.
+
+	Remarks:
+		This may not be called with untrusted input or it'll result in arbitrary code execution, especially on Windows.
+*/
+int open_file(const char *path);
+
 void swap_endian(void *data, unsigned elem_size, unsigned num);
 
 typedef void (*DBG_LOGGER)(const char *line, void *user);
 typedef void (*DBG_LOGGER_FINISH)(void *user);
 void dbg_logger(DBG_LOGGER logger, DBG_LOGGER_FINISH finish, void *user);
 
-void dbg_logger_stdout(void);
-void dbg_logger_debugger(void);
+void dbg_logger_stdout();
+void dbg_logger_debugger();
 void dbg_logger_file(const char *filename);
 
 typedef struct
 {
-	int sent_packets;
-	int sent_bytes;
-	int recv_packets;
-	int recv_bytes;
+	uint64_t sent_packets;
+	uint64_t sent_bytes;
+	uint64_t recv_packets;
+	uint64_t recv_bytes;
 } NETSTATS;
 
 void net_stats(NETSTATS *stats);
@@ -1854,14 +1891,14 @@ int str_utf8_to_skeleton(const char *str, int *buf, int buf_len);
 		Compares two strings for visual appearance.
 
 	Parameters:
-		a - String to compare.
-		b - String to compare.
+		str1 - String to compare.
+		str2 - String to compare.
 
 	Returns:
 		0 if the strings are confusable.
 		!=0 otherwise.
 */
-int str_utf8_comp_confusable(const char *a, const char *b);
+int str_utf8_comp_confusable(const char *str1, const char *str2);
 
 /*
 	Function: str_utf8_tolower
@@ -1959,13 +1996,13 @@ const char *str_utf8_skip_whitespaces(const char *str);
 		the string in-place.
 
 	Parameters:
-		str - Pointer to the string.
+		param - Pointer to the string.
 
 	Remarks:
 		- The strings are treated as zero-terminated strings.
 		- The string is modified in-place.
 */
-void str_utf8_trim_right(char *str);
+void str_utf8_trim_right(char *param);
 
 /*
 	Function: str_utf8_rewind
@@ -1983,6 +2020,19 @@ void str_utf8_trim_right(char *str);
 */
 int str_utf8_rewind(const char *str, int cursor);
 
+/*
+	Function: str_utf8_fix_truncation
+		Fixes truncation of a Unicode character at the end of a UTF-8
+		string.
+
+	Returns:
+		The new string length.
+
+	Parameters:
+		str - utf8 string
+*/
+int str_utf8_fix_truncation(char *str);
+
 /*
 	Function: str_utf8_forward
 		Moves a cursor forwards in an utf8 string
@@ -2062,31 +2112,35 @@ int str_utf16le_encode(char *ptr, int chr);
 int str_utf8_check(const char *str);
 
 /*
-	Function: str_utf8_copy
-		Copies a utf8 string to a buffer.
+	Function: str_utf8_stats
+		Determines the byte size and utf8 character count of a utf8 string.
 
 	Parameters:
-		dst - Pointer to a buffer that shall receive the string.
-		src - utf8 string to be copied.
-		dst_size - Size of the buffer dst.
+		str - Pointer to the string.
+		max_size - Maximum number of bytes to count.
+		max_count - Maximum number of utf8 characters to count.
+		size - Pointer to store size (number of non-zero bytes) of the string.
+		count - Pointer to store count of utf8 characters of the string.
 
 	Remarks:
-		- The strings are treated as zero-terminated strings.
-		- Guarantees that dst string will contain zero-termination.
-		- Guarantees that dst always contains a valid utf8 string.
+		- The string is treated as zero-terminated utf8 string.
+		- It's the user's responsibility to make sure the bounds are aligned.
 */
-void str_utf8_copy(char *dst, const char *src, int dst_size);
+void str_utf8_stats(const char *str, int max_size, int max_count, int *size, int *count);
 
 /*
 	Function: str_next_token
 		Writes the next token after str into buf, returns the rest of the string.
+
 	Parameters:
 		str - Pointer to string.
 		delim - Delimiter for tokenization.
 		buffer - Buffer to store token in.
 		buffer_size - Size of the buffer.
+
 	Returns:
 		Pointer to rest of the string.
+
 	Remarks:
 		- The token is always null-terminated.
 */
@@ -2107,6 +2161,52 @@ const char *str_next_token(const char *str, const char *delim, char *buffer, int
 */
 int str_in_list(const char *list, const char *delim, const char *needle);
 
+/*
+	Function: bytes_be_to_int
+		Packs 4 big endian bytes into an int
+
+	Returns:
+		The packed int
+
+	Remarks:
+		- Assumes the passed array is 4 bytes
+		- Assumes int is 4 bytes
+*/
+int bytes_be_to_int(const unsigned char *bytes);
+
+/*
+	Function: int_to_bytes_be
+		Packs an int into 4 big endian bytes
+
+	Remarks:
+		- Assumes the passed array is 4 bytes
+		- Assumes int is 4 bytes
+*/
+void int_to_bytes_be(unsigned char *bytes, int value);
+
+/*
+	Function: bytes_be_to_uint
+		Packs 4 big endian bytes into an unsigned
+
+	Returns:
+		The packed unsigned
+
+	Remarks:
+		- Assumes the passed array is 4 bytes
+		- Assumes unsigned is 4 bytes
+*/
+unsigned bytes_be_to_uint(const unsigned char *bytes);
+
+/*
+	Function: uint_to_bytes_be
+		Packs an unsigned into 4 big endian bytes
+
+	Remarks:
+		- Assumes the passed array is 4 bytes
+		- Assumes unsigned is 4 bytes
+*/
+void uint_to_bytes_be(unsigned char *bytes, unsigned value);
+
 /*
 	Function: pid
 		Returns the pid of the current process.
@@ -2114,7 +2214,33 @@ int str_in_list(const char *list, const char *delim, const char *needle);
 	Returns:
 		pid of the current process
 */
-int pid(void);
+int pid();
+
+/*
+	Function: cmdline_fix
+		Fixes the command line arguments to be encoded in UTF-8 on all
+		systems.
+
+	Parameters:
+		argc - A pointer to the argc parameter that was passed to the main function.
+		argv - A pointer to the argv parameter that was passed to the main function.
+
+	Remarks:
+		- You need to call cmdline_free once you're no longer using the
+		results.
+*/
+void cmdline_fix(int *argc, const char ***argv);
+
+/*
+	Function: cmdline_free
+		Frees memory that was allocated by cmdline_fix.
+
+	Parameters:
+		argc - The argc obtained from cmdline_fix.
+		argv - The argv obtained from cmdline_fix.
+
+*/
+void cmdline_free(int argc, const char **argv);
 
 #if defined(CONF_FAMILY_WINDOWS)
 typedef void *PROCESS;
@@ -2144,16 +2270,6 @@ PROCESS shell_execute(const char *file);
 */
 int kill_process(PROCESS process);
 
-/*
-	Function: os_is_winxp_or_lower
-		Checks whether the program runs on Windows XP or lower.
-
-	Returns:
-		1 - Windows XP or lower.
-		0 - Higher Windows version, Linux, macOS, etc.
-*/
-int os_is_winxp_or_lower(void);
-
 /*
 	Function: generate_password
 		Generates a null-terminated password of length `2 *
@@ -2176,7 +2292,7 @@ void generate_password(char *buffer, unsigned length, unsigned short *random, un
 		0 - Initialization succeeded.
 		1 - Initialization failed.
 */
-int secure_random_init(void);
+int secure_random_init();
 
 /*
 	Function: secure_random_password
@@ -2207,7 +2323,7 @@ void secure_random_fill(void *bytes, unsigned length);
 	Function: secure_rand
 		Returns random int (replacement for rand()).
 */
-int secure_rand(void);
+int secure_rand();
 
 /*
 	Function: secure_rand_below
@@ -2219,7 +2335,30 @@ int secure_rand(void);
 */
 int secure_rand_below(int below);
 
-#ifdef __cplusplus
+/*
+	Function: set_console_msg_color
+		Sets the console color.
+
+	Parameters:
+		rgb - If NULL it will reset the console color to default, else it will transform the rgb color to a console color
+*/
+void set_console_msg_color(const void *rgbvoid);
+
+/*
+	Function: os_version_str
+		Returns a human-readable version string of the operating system
+
+	Parameters:
+		version - Buffer to use for the output.
+		length - Length of the output buffer.
+
+	Returns:
+		0 - Success in getting the version.
+		1 - Failure in getting the version.
+*/
+int os_version_str(char *version, int length);
+
+#if defined(__cplusplus)
 }
 #endif
 
