@@ -31,33 +31,34 @@
 #endif
 
 #include "graphics_threaded.h"
+#include "graphics_threaded_null.h"
 
 static CVideoMode g_aFakeModes[] = {
-	{8192, 4320, 8192, 4320, 8, 8, 8}, {7680, 4320, 7680, 4320, 8, 8, 8}, {5120, 2880, 5120, 2880, 8, 8, 8},
-	{4096, 2160, 4096, 2160, 8, 8, 8}, {3840, 2160, 3840, 2160, 8, 8, 8}, {2560, 1440, 2560, 1440, 8, 8, 8},
-	{2048, 1536, 2048, 1536, 8, 8, 8}, {1920, 2400, 1920, 2400, 8, 8, 8}, {1920, 1440, 1920, 1440, 8, 8, 8},
-	{1920, 1200, 1920, 1200, 8, 8, 8}, {1920, 1080, 1920, 1080, 8, 8, 8}, {1856, 1392, 1856, 1392, 8, 8, 8},
-	{1800, 1440, 1800, 1440, 8, 8, 8}, {1792, 1344, 1792, 1344, 8, 8, 8}, {1680, 1050, 1680, 1050, 8, 8, 8},
-	{1600, 1200, 1600, 1200, 8, 8, 8}, {1600, 1000, 1600, 1000, 8, 8, 8}, {1440, 1050, 1440, 1050, 8, 8, 8},
-	{1440, 900, 1440, 900, 8, 8, 8}, {1400, 1050, 1400, 1050, 8, 8, 8}, {1368, 768, 1368, 768, 8, 8, 8},
-	{1280, 1024, 1280, 1024, 8, 8, 8}, {1280, 960, 1280, 960, 8, 8, 8}, {1280, 800, 1280, 800, 8, 8, 8},
-	{1280, 768, 1280, 768, 8, 8, 8}, {1152, 864, 1152, 864, 8, 8, 8}, {1024, 768, 1024, 768, 8, 8, 8},
-	{1024, 600, 1024, 600, 8, 8, 8}, {800, 600, 800, 600, 8, 8, 8}, {768, 576, 768, 576, 8, 8, 8},
-	{720, 400, 720, 400, 8, 8, 8}, {640, 480, 640, 480, 8, 8, 8}, {400, 300, 400, 300, 8, 8, 8},
-	{320, 240, 320, 240, 8, 8, 8},
+	{8192, 4320, 8192, 4320, 0, 8, 8, 8, 0}, {7680, 4320, 7680, 4320, 0, 8, 8, 8, 0}, {5120, 2880, 5120, 2880, 0, 8, 8, 8, 0},
+	{4096, 2160, 4096, 2160, 0, 8, 8, 8, 0}, {3840, 2160, 3840, 2160, 0, 8, 8, 8, 0}, {2560, 1440, 2560, 1440, 0, 8, 8, 8, 0},
+	{2048, 1536, 2048, 1536, 0, 8, 8, 8, 0}, {1920, 2400, 1920, 2400, 0, 8, 8, 8, 0}, {1920, 1440, 1920, 1440, 0, 8, 8, 8, 0},
+	{1920, 1200, 1920, 1200, 0, 8, 8, 8, 0}, {1920, 1080, 1920, 1080, 0, 8, 8, 8, 0}, {1856, 1392, 1856, 1392, 0, 8, 8, 8, 0},
+	{1800, 1440, 1800, 1440, 0, 8, 8, 8, 0}, {1792, 1344, 1792, 1344, 0, 8, 8, 8, 0}, {1680, 1050, 1680, 1050, 0, 8, 8, 8, 0},
+	{1600, 1200, 1600, 1200, 0, 8, 8, 8, 0}, {1600, 1000, 1600, 1000, 0, 8, 8, 8, 0}, {1440, 1050, 1440, 1050, 0, 8, 8, 8, 0},
+	{1440, 900, 1440, 900, 0, 8, 8, 8, 0}, {1400, 1050, 1400, 1050, 0, 8, 8, 8, 0}, {1368, 768, 1368, 768, 0, 8, 8, 8, 0},
+	{1280, 1024, 1280, 1024, 0, 8, 8, 8, 0}, {1280, 960, 1280, 960, 0, 8, 8, 8, 0}, {1280, 800, 1280, 800, 0, 8, 8, 8, 0},
+	{1280, 768, 1280, 768, 0, 8, 8, 8, 0}, {1152, 864, 1152, 864, 0, 8, 8, 8, 0}, {1024, 768, 1024, 768, 0, 8, 8, 8, 0},
+	{1024, 600, 1024, 600, 0, 8, 8, 8, 0}, {800, 600, 800, 600, 0, 8, 8, 8, 0}, {768, 576, 768, 576, 0, 8, 8, 8, 0},
+	{720, 400, 720, 400, 0, 8, 8, 8, 0}, {640, 480, 640, 480, 0, 8, 8, 8, 0}, {400, 300, 400, 300, 0, 8, 8, 8, 0},
+	{320, 240, 320, 240, 0, 8, 8, 8, 0},
 
-	{8192, 4320, 8192, 4320, 5, 6, 5}, {7680, 4320, 7680, 4320, 5, 6, 5}, {5120, 2880, 5120, 2880, 5, 6, 5},
-	{4096, 2160, 4096, 2160, 5, 6, 5}, {3840, 2160, 3840, 2160, 5, 6, 5}, {2560, 1440, 2560, 1440, 5, 6, 5},
-	{2048, 1536, 2048, 1536, 5, 6, 5}, {1920, 2400, 1920, 2400, 5, 6, 5}, {1920, 1440, 1920, 1440, 5, 6, 5},
-	{1920, 1200, 1920, 1200, 5, 6, 5}, {1920, 1080, 1920, 1080, 5, 6, 5}, {1856, 1392, 1856, 1392, 5, 6, 5},
-	{1800, 1440, 1800, 1440, 5, 6, 5}, {1792, 1344, 1792, 1344, 5, 6, 5}, {1680, 1050, 1680, 1050, 5, 6, 5},
-	{1600, 1200, 1600, 1200, 5, 6, 5}, {1600, 1000, 1600, 1000, 5, 6, 5}, {1440, 1050, 1440, 1050, 5, 6, 5},
-	{1440, 900, 1440, 900, 5, 6, 5}, {1400, 1050, 1400, 1050, 5, 6, 5}, {1368, 768, 1368, 768, 5, 6, 5},
-	{1280, 1024, 1280, 1024, 5, 6, 5}, {1280, 960, 1280, 960, 5, 6, 5}, {1280, 800, 1280, 800, 5, 6, 5},
-	{1280, 768, 1280, 768, 5, 6, 5}, {1152, 864, 1152, 864, 5, 6, 5}, {1024, 768, 1024, 768, 5, 6, 5},
-	{1024, 600, 1024, 600, 5, 6, 5}, {800, 600, 800, 600, 5, 6, 5}, {768, 576, 768, 576, 5, 6, 5},
-	{720, 400, 720, 400, 5, 6, 5}, {640, 480, 640, 480, 5, 6, 5}, {400, 300, 400, 300, 5, 6, 5},
-	{320, 240, 320, 240, 5, 6, 5}};
+	{8192, 4320, 8192, 4320, 0, 5, 6, 5, 0}, {7680, 4320, 7680, 4320, 0, 5, 6, 5, 0}, {5120, 2880, 5120, 2880, 0, 5, 6, 5, 0},
+	{4096, 2160, 4096, 2160, 0, 5, 6, 5, 0}, {3840, 2160, 3840, 2160, 0, 5, 6, 5, 0}, {2560, 1440, 2560, 1440, 0, 5, 6, 5, 0},
+	{2048, 1536, 2048, 1536, 0, 5, 6, 5, 0}, {1920, 2400, 1920, 2400, 0, 5, 6, 5, 0}, {1920, 1440, 1920, 1440, 0, 5, 6, 5, 0},
+	{1920, 1200, 1920, 1200, 0, 5, 6, 5, 0}, {1920, 1080, 1920, 1080, 0, 5, 6, 5, 0}, {1856, 1392, 1856, 1392, 0, 5, 6, 5, 0},
+	{1800, 1440, 1800, 1440, 0, 5, 6, 5, 0}, {1792, 1344, 1792, 1344, 0, 5, 6, 5, 0}, {1680, 1050, 1680, 1050, 0, 5, 6, 5, 0},
+	{1600, 1200, 1600, 1200, 0, 5, 6, 5, 0}, {1600, 1000, 1600, 1000, 0, 5, 6, 5, 0}, {1440, 1050, 1440, 1050, 0, 5, 6, 5, 0},
+	{1440, 900, 1440, 900, 0, 5, 6, 5, 0}, {1400, 1050, 1400, 1050, 0, 5, 6, 5, 0}, {1368, 768, 1368, 768, 0, 5, 6, 5, 0},
+	{1280, 1024, 1280, 1024, 0, 5, 6, 5, 0}, {1280, 960, 1280, 960, 0, 5, 6, 5, 0}, {1280, 800, 1280, 800, 0, 5, 6, 5, 0},
+	{1280, 768, 1280, 768, 0, 5, 6, 5, 0}, {1152, 864, 1152, 864, 0, 5, 6, 5, 0}, {1024, 768, 1024, 768, 0, 5, 6, 5, 0},
+	{1024, 600, 1024, 600, 0, 5, 6, 5, 0}, {800, 600, 800, 600, 0, 5, 6, 5, 0}, {768, 576, 768, 576, 0, 5, 6, 5, 0},
+	{720, 400, 720, 400, 0, 5, 6, 5, 0}, {640, 480, 640, 480, 0, 5, 6, 5, 0}, {400, 300, 400, 300, 0, 5, 6, 5, 0},
+	{320, 240, 320, 240, 0, 5, 6, 5, 0}};
 
 void CGraphics_Threaded::FlushVertices(bool KeepVertices)
 {
@@ -146,6 +147,7 @@ CGraphics_Threaded::CGraphics_Threaded()
 
 	m_ScreenWidth = -1;
 	m_ScreenHeight = -1;
+	m_ScreenRefreshRate = -1;
 
 	m_Rotation = 0;
 	m_Drawing = 0;
@@ -262,29 +264,24 @@ void CGraphics_Threaded::LinesDraw(const CLineItem *pArray, int Num)
 	AddVertices(2 * Num);
 }
 
-int CGraphics_Threaded::UnloadTexture(CTextureHandle Index)
+int CGraphics_Threaded::UnloadTexture(CTextureHandle *pIndex)
 {
-	if(Index == m_InvalidTexture)
+	if(pIndex->Id() == m_InvalidTexture.Id())
 		return 0;
 
-	if(Index < 0)
+	if(!pIndex->IsValid())
 		return 0;
 
 	CCommandBuffer::SCommand_Texture_Destroy Cmd;
-	Cmd.m_Slot = Index;
+	Cmd.m_Slot = pIndex->Id();
 	AddCmd(
 		Cmd, [] { return true; }, "failed to unload texture.");
 
-	m_TextureIndices[Index] = m_FirstFreeTexture;
-	m_FirstFreeTexture = Index;
-	return 0;
-}
+	m_TextureIndices[pIndex->Id()] = m_FirstFreeTexture;
+	m_FirstFreeTexture = pIndex->Id();
 
-int CGraphics_Threaded::UnloadTextureNew(CTextureHandle &TextureHandle)
-{
-	int Ret = UnloadTexture(TextureHandle);
-	TextureHandle = IGraphics::CTextureHandle();
-	return Ret;
+	pIndex->Invalidate();
+	return 0;
 }
 
 static int ImageFormatToTexFormat(int Format)
@@ -311,7 +308,7 @@ static int ImageFormatToPixelSize(int Format)
 int CGraphics_Threaded::LoadTextureRawSub(CTextureHandle TextureID, int x, int y, int Width, int Height, int Format, const void *pData)
 {
 	CCommandBuffer::SCommand_Texture_Update Cmd;
-	Cmd.m_Slot = TextureID;
+	Cmd.m_Slot = TextureID.Id();
 	Cmd.m_X = x;
 	Cmd.m_Y = y;
 	Cmd.m_Width = Width;
@@ -483,7 +480,7 @@ IGraphics::CTextureHandle CGraphics_Threaded::LoadTextureRaw(int Width, int Heig
 IGraphics::CTextureHandle CGraphics_Threaded::LoadTexture(const char *pFilename, int StorageType, int StoreFormat, int Flags)
 {
 	int l = str_length(pFilename);
-	int ID;
+	IGraphics::CTextureHandle ID;
 	CImageInfo Img;
 
 	if(l < 3)
@@ -495,9 +492,9 @@ IGraphics::CTextureHandle CGraphics_Threaded::LoadTexture(const char *pFilename,
 
 		ID = LoadTextureRaw(Img.m_Width, Img.m_Height, Img.m_Format, Img.m_pData, StoreFormat, Flags, pFilename);
 		free(Img.m_pData);
-		if(ID != m_InvalidTexture && g_Config.m_Debug)
+		if(ID.Id() != m_InvalidTexture.Id() && g_Config.m_Debug)
 			dbg_msg("graphics/texture", "loaded %s", pFilename);
-		return CreateTextureHandle(ID);
+		return ID;
 	}
 
 	return m_InvalidTexture;
@@ -505,46 +502,42 @@ IGraphics::CTextureHandle CGraphics_Threaded::LoadTexture(const char *pFilename,
 
 int CGraphics_Threaded::LoadPNG(CImageInfo *pImg, const char *pFilename, int StorageType)
 {
-	char aCompleteFilename[512];
-	unsigned char *pBuffer;
-	png_t Png; // ignore_convention
+	char aCompleteFilename[IO_MAX_PATH_LENGTH];
 
 	// open file for reading
 	IOHANDLE File = m_pStorage->OpenFile(pFilename, IOFLAG_READ, StorageType, aCompleteFilename, sizeof(aCompleteFilename));
-	if(File)
-		io_close(File);
-	else
+	if(!File)
 	{
 		dbg_msg("game/png", "failed to open file. filename='%s'", pFilename);
 		return 0;
 	}
 
-	int Error = png_open_file(&Png, aCompleteFilename); // ignore_convention
+	png_t Png; // ignore_convention
+	int Error = png_open_read(&Png, 0, File); // ignore_convention
 	if(Error != PNG_NO_ERROR)
 	{
 		dbg_msg("game/png", "failed to open file. filename='%s', pnglite: %s", aCompleteFilename, png_error_string(Error));
-		if(Error != PNG_FILE_ERROR)
-			png_close_file(&Png); // ignore_convention
+		io_close(File);
 		return 0;
 	}
 
 	if(Png.depth != 8 || (Png.color_type != PNG_TRUECOLOR && Png.color_type != PNG_TRUECOLOR_ALPHA)) // ignore_convention
 	{
 		dbg_msg("game/png", "invalid format. filename='%s'", aCompleteFilename);
-		png_close_file(&Png); // ignore_convention
+		io_close(File);
 		return 0;
 	}
 
-	pBuffer = (unsigned char *)malloc((size_t)Png.width * Png.height * Png.bpp); // ignore_convention
+	unsigned char *pBuffer = (unsigned char *)malloc((size_t)Png.width * Png.height * Png.bpp); // ignore_convention
 	Error = png_get_data(&Png, pBuffer); // ignore_convention
 	if(Error != PNG_NO_ERROR)
 	{
 		dbg_msg("game/png", "failed to read image. filename='%s', pnglite: %s", aCompleteFilename, png_error_string(Error));
 		free(pBuffer);
-		png_close_file(&Png); // ignore_convention
+		io_close(File);
 		return 0;
 	}
-	png_close_file(&Png); // ignore_convention
+	io_close(File);
 
 	pImg->m_Width = Png.width; // ignore_convention
 	pImg->m_Height = Png.height; // ignore_convention
@@ -670,16 +663,20 @@ void CGraphics_Threaded::ScreenshotDirect()
 		png_t Png; // ignore_convention
 
 		IOHANDLE File = m_pStorage->OpenFile(m_aScreenshotName, IOFLAG_WRITE, IStorage::TYPE_SAVE, aWholePath, sizeof(aWholePath));
-		if(File)
-			io_close(File);
-
-		// save png
-		char aBuf[256];
-		str_format(aBuf, sizeof(aBuf), "saved screenshot to '%s'", aWholePath);
-		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "client", aBuf, ColorRGBA{1.0f, 0.6f, 0.3f, 1.0f});
-		png_open_file_write(&Png, aWholePath); // ignore_convention
-		png_set_data(&Png, Image.m_Width, Image.m_Height, 8, PNG_TRUECOLOR_ALPHA, (unsigned char *)Image.m_pData); // ignore_convention
-		png_close_file(&Png); // ignore_convention
+		if(!File)
+		{
+			dbg_msg("game/screenshot", "failed to open file. filename='%s'", aWholePath);
+		}
+		else
+		{
+			// save png
+			char aBuf[256];
+			str_format(aBuf, sizeof(aBuf), "saved screenshot to '%s'", aWholePath);
+			m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "client", aBuf, ColorRGBA(1.0f, 0.6f, 0.3f, 1.0f));
+			png_open_write(&Png, 0, File); // ignore_convention
+			png_set_data(&Png, Image.m_Width, Image.m_Height, 8, PNG_TRUECOLOR_ALPHA, (unsigned char *)Image.m_pData); // ignore_convention
+			io_close(File); // ignore_convention
+		}
 
 		free(Image.m_pData);
 	}
@@ -688,7 +685,7 @@ void CGraphics_Threaded::ScreenshotDirect()
 void CGraphics_Threaded::TextureSet(CTextureHandle TextureID)
 {
 	dbg_assert(m_Drawing == 0, "called Graphics()->TextureSet within begin");
-	m_State.m_Texture = TextureID;
+	m_State.m_Texture = TextureID.Id();
 }
 
 void CGraphics_Threaded::Clear(float r, float g, float b)
@@ -1249,13 +1246,13 @@ void CGraphics_Threaded::RenderText(int BufferContainerIndex, int TextQuadNum, i
 	}
 }
 
-int CGraphics_Threaded::CreateQuadContainer()
+int CGraphics_Threaded::CreateQuadContainer(bool AutomaticUpload)
 {
 	int Index = -1;
 	if(m_FirstFreeQuadContainer == -1)
 	{
 		Index = m_QuadContainers.size();
-		m_QuadContainers.push_back(SQuadContainer());
+		m_QuadContainers.push_back(SQuadContainer(AutomaticUpload));
 	}
 	else
 	{
@@ -1267,12 +1264,18 @@ int CGraphics_Threaded::CreateQuadContainer()
 	return Index;
 }
 
+void CGraphics_Threaded::QuadContainerChangeAutomaticUpload(int ContainerIndex, bool AutomaticUpload)
+{
+	SQuadContainer &Container = m_QuadContainers[ContainerIndex];
+	Container.m_AutomaticUpload = AutomaticUpload;
+}
+
 void CGraphics_Threaded::QuadContainerUpload(int ContainerIndex)
 {
 	if(IsQuadContainerBufferingEnabled())
 	{
 		SQuadContainer &Container = m_QuadContainers[ContainerIndex];
-		if(Container.m_Quads.size() > 0)
+		if(!Container.m_Quads.empty())
 		{
 			if(Container.m_QuadBufferObjectIndex == -1)
 			{
@@ -1363,7 +1366,8 @@ void CGraphics_Threaded::QuadContainerAddQuads(int ContainerIndex, CQuadItem *pA
 		}
 	}
 
-	QuadContainerUpload(ContainerIndex);
+	if(Container.m_AutomaticUpload)
+		QuadContainerUpload(ContainerIndex);
 }
 
 void CGraphics_Threaded::QuadContainerAddQuads(int ContainerIndex, CFreeformItem *pArray, int Num)
@@ -1399,7 +1403,8 @@ void CGraphics_Threaded::QuadContainerAddQuads(int ContainerIndex, CFreeformItem
 		SetColor(&Quad.m_aVertices[3], 2);
 	}
 
-	QuadContainerUpload(ContainerIndex);
+	if(Container.m_AutomaticUpload)
+		QuadContainerUpload(ContainerIndex);
 }
 
 void CGraphics_Threaded::QuadContainerReset(int ContainerIndex)
@@ -1428,7 +1433,7 @@ void CGraphics_Threaded::RenderQuadContainer(int ContainerIndex, int QuadDrawNum
 	RenderQuadContainer(ContainerIndex, 0, QuadDrawNum);
 }
 
-void CGraphics_Threaded::RenderQuadContainer(int ContainerIndex, int QuadOffset, int QuadDrawNum)
+void CGraphics_Threaded::RenderQuadContainer(int ContainerIndex, int QuadOffset, int QuadDrawNum, bool ChangeWrapMode)
 {
 	SQuadContainer &Container = m_QuadContainers[ContainerIndex];
 
@@ -1443,7 +1448,8 @@ void CGraphics_Threaded::RenderQuadContainer(int ContainerIndex, int QuadOffset,
 		if(Container.m_QuadBufferContainerIndex == -1)
 			return;
 
-		WrapClamp();
+		if(ChangeWrapMode)
+			WrapClamp();
 
 		CCommandBuffer::SCommand_RenderQuadContainer Cmd;
 		Cmd.m_State = m_State;
@@ -1479,7 +1485,8 @@ void CGraphics_Threaded::RenderQuadContainer(int ContainerIndex, int QuadOffset,
 			m_NumVertices += 4 * QuadDrawNum;
 		}
 		m_Drawing = DRAWING_QUADS;
-		WrapClamp();
+		if(ChangeWrapMode)
+			WrapClamp();
 		FlushVertices(false);
 		m_Drawing = 0;
 	}
@@ -2070,20 +2077,28 @@ int CGraphics_Threaded::IssueInit()
 {
 	int Flags = 0;
 
+	bool IsPurlyWindowed = g_Config.m_GfxFullscreen == 0;
+	bool IsExclusiveFullscreen = g_Config.m_GfxFullscreen == 1;
+	bool IsDesktopFullscreen = g_Config.m_GfxFullscreen == 2;
+#ifndef CONF_FAMILY_WINDOWS
+	//  special mode for windows only
+	IsDesktopFullscreen |= g_Config.m_GfxFullscreen == 3;
+#endif
+
 	if(g_Config.m_GfxBorderless)
 		Flags |= IGraphicsBackend::INITFLAG_BORDERLESS;
-	if(g_Config.m_GfxFullscreen == 1)
+	if(IsExclusiveFullscreen)
 		Flags |= IGraphicsBackend::INITFLAG_FULLSCREEN;
-	else if(g_Config.m_GfxFullscreen == 2)
+	else if(IsDesktopFullscreen)
 		Flags |= IGraphicsBackend::INITFLAG_DESKTOP_FULLSCREEN;
+	if(IsPurlyWindowed || IsExclusiveFullscreen || IsDesktopFullscreen)
+		Flags |= IGraphicsBackend::INITFLAG_RESIZABLE;
 	if(g_Config.m_GfxVsync)
 		Flags |= IGraphicsBackend::INITFLAG_VSYNC;
 	if(g_Config.m_GfxHighdpi)
 		Flags |= IGraphicsBackend::INITFLAG_HIGHDPI;
-	if(g_Config.m_GfxResizable)
-		Flags |= IGraphicsBackend::INITFLAG_RESIZABLE;
 
-	int r = m_pBackend->Init("DDNet Client", &g_Config.m_GfxScreen, &g_Config.m_GfxScreenWidth, &g_Config.m_GfxScreenHeight, g_Config.m_GfxFsaaSamples, Flags, &g_Config.m_GfxDesktopWidth, &g_Config.m_GfxDesktopHeight, &m_ScreenWidth, &m_ScreenHeight, m_pStorage);
+	int r = m_pBackend->Init("DDNet Client", &g_Config.m_GfxScreen, &g_Config.m_GfxScreenWidth, &g_Config.m_GfxScreenHeight, &g_Config.m_GfxScreenRefreshRate, g_Config.m_GfxFsaaSamples, Flags, &g_Config.m_GfxDesktopWidth, &g_Config.m_GfxDesktopHeight, &m_ScreenWidth, &m_ScreenHeight, m_pStorage);
 	AddBackEndWarningIfExists();
 	m_IsNewOpenGL = m_pBackend->IsNewOpenGL();
 	m_OpenGLTileBufferingEnabled = m_IsNewOpenGL || m_pBackend->HasTileBuffering();
@@ -2092,6 +2107,7 @@ int CGraphics_Threaded::IssueInit()
 	m_OpenGLTextBufferingEnabled = m_IsNewOpenGL || (m_OpenGLQuadContainerBufferingEnabled && m_pBackend->HasTextBuffering());
 	m_OpenGLHasTextureArrays = m_IsNewOpenGL || m_pBackend->Has2DTextureArrays();
 	m_ScreenHiDPIScale = m_ScreenWidth / (float)g_Config.m_GfxScreenWidth;
+	m_ScreenRefreshRate = g_Config.m_GfxScreenRefreshRate;
 	return r;
 }
 
@@ -2244,6 +2260,7 @@ int CGraphics_Threaded::Init()
 	{
 		FakeMode.m_WindowWidth = FakeMode.m_CanvasWidth / m_ScreenHiDPIScale;
 		FakeMode.m_WindowHeight = FakeMode.m_CanvasHeight / m_ScreenHiDPIScale;
+		FakeMode.m_RefreshRate = g_Config.m_GfxScreenRefreshRate;
 	}
 
 	// create command buffers
@@ -2303,36 +2320,83 @@ void CGraphics_Threaded::Maximize()
 	m_pBackend->Maximize();
 }
 
-void CGraphics_Threaded::SetWindowParams(int FullscreenMode, bool IsBorderless)
+void CGraphics_Threaded::SetWindowParams(int FullscreenMode, bool IsBorderless, bool AllowResizing)
 {
-	m_pBackend->SetWindowParams(FullscreenMode, IsBorderless);
+	m_pBackend->SetWindowParams(FullscreenMode, IsBorderless, AllowResizing);
+	CVideoMode CurMode;
+	m_pBackend->GetCurrentVideoMode(CurMode, m_ScreenHiDPIScale, g_Config.m_GfxDesktopWidth, g_Config.m_GfxDesktopHeight, g_Config.m_GfxScreen);
+	GotResized(CurMode.m_WindowWidth, CurMode.m_WindowHeight, CurMode.m_RefreshRate);
 }
 
 bool CGraphics_Threaded::SetWindowScreen(int Index)
 {
-	return m_pBackend->SetWindowScreen(Index);
+	if(!m_pBackend->SetWindowScreen(Index))
+	{
+		return false;
+	}
+
+	m_pBackend->GetViewportSize(m_ScreenWidth, m_ScreenHeight);
+	m_ScreenHiDPIScale = m_ScreenWidth / (float)g_Config.m_GfxScreenWidth;
+	return true;
 }
 
-void CGraphics_Threaded::Resize(int w, int h, bool SetWindowSize)
+void CGraphics_Threaded::Move(int x, int y)
 {
 #if defined(CONF_VIDEORECORDER)
 	if(IVideo::Current() && IVideo::Current()->IsRecording())
 		return;
 #endif
 
-	if(WindowWidth() == w && WindowHeight() == h)
+	// Only handling CurScreen != m_GfxScreen doesn't work reliably
+	const int CurScreen = m_pBackend->GetWindowScreen();
+	m_pBackend->UpdateDisplayMode(CurScreen);
+	m_pBackend->GetViewportSize(m_ScreenWidth, m_ScreenHeight);
+	m_ScreenHiDPIScale = m_ScreenWidth / (float)g_Config.m_GfxScreenWidth;
+}
+
+void CGraphics_Threaded::Resize(int w, int h, int RefreshRate)
+{
+#if defined(CONF_VIDEORECORDER)
+	if(IVideo::Current() && IVideo::Current()->IsRecording())
+		return;
+#endif
+
+	if(WindowWidth() == w && WindowHeight() == h && RefreshRate == m_ScreenRefreshRate)
 		return;
 
-	if(SetWindowSize)
-		m_pBackend->ResizeWindow(w, h);
+	// if the size is changed manually, only set the window resize, a window size changed event is triggered anyway
+	if(m_pBackend->ResizeWindow(w, h, RefreshRate))
+	{
+		CVideoMode CurMode;
+		m_pBackend->GetCurrentVideoMode(CurMode, m_ScreenHiDPIScale, g_Config.m_GfxDesktopWidth, g_Config.m_GfxDesktopHeight, g_Config.m_GfxScreen);
+		GotResized(w, h, RefreshRate);
+	}
+}
 
+void CGraphics_Threaded::GotResized(int w, int h, int RefreshRate)
+{
+#if defined(CONF_VIDEORECORDER)
+	if(IVideo::Current() && IVideo::Current()->IsRecording())
+		return;
+#endif
+
+	// if RefreshRate is -1 use the current config refresh rate
+	if(RefreshRate == -1)
+		RefreshRate = g_Config.m_GfxScreenRefreshRate;
+
+	// if the size change event is triggered, set all parameters and change the viewport
 	m_pBackend->GetViewportSize(m_ScreenWidth, m_ScreenHeight);
 
 	// adjust the viewport to only allow certain aspect ratios
 	if(m_ScreenHeight > 4 * m_ScreenWidth / 5)
 		m_ScreenHeight = 4 * m_ScreenWidth / 5;
-	if(m_ScreenWidth > 21 * m_ScreenHeight / 9)
-		m_ScreenWidth = 21 * m_ScreenHeight / 9;
+
+	m_ScreenRefreshRate = RefreshRate;
+
+	g_Config.m_GfxScreenWidth = w;
+	g_Config.m_GfxScreenHeight = h;
+	g_Config.m_GfxScreenRefreshRate = m_ScreenRefreshRate;
+	m_ScreenHiDPIScale = m_ScreenWidth / (float)g_Config.m_GfxScreenWidth;
 
 	CCommandBuffer::SCommand_Update_Viewport Cmd;
 	Cmd.m_X = 0;
@@ -2346,7 +2410,7 @@ void CGraphics_Threaded::Resize(int w, int h, bool SetWindowSize)
 		return;
 	}
 
-	// kick the command buffer
+	// kick the command buffer and wait
 	KickCommandBuffer();
 	WaitForIdle();
 
@@ -2364,6 +2428,34 @@ int CGraphics_Threaded::GetWindowScreen()
 	return m_pBackend->GetWindowScreen();
 }
 
+void CGraphics_Threaded::WindowDestroyNtf(uint32_t WindowID)
+{
+	m_pBackend->WindowDestroyNtf(WindowID);
+
+	CCommandBuffer::SCommand_WindowDestroyNtf Cmd;
+	Cmd.m_WindowID = WindowID;
+
+	if(!AddCmd(
+		   Cmd, [] { return true; }, "failed to add window destroy notify command"))
+	{
+		return;
+	}
+}
+
+void CGraphics_Threaded::WindowCreateNtf(uint32_t WindowID)
+{
+	m_pBackend->WindowCreateNtf(WindowID);
+
+	CCommandBuffer::SCommand_WindowCreateNtf Cmd;
+	Cmd.m_WindowID = WindowID;
+
+	if(!AddCmd(
+		   Cmd, [] { return true; }, "failed to add window create notify command"))
+	{
+		return;
+	}
+}
+
 int CGraphics_Threaded::WindowActive()
 {
 	return m_pBackend->WindowActive();
@@ -2440,6 +2532,11 @@ void CGraphics_Threaded::Swap()
 
 	// kick the command buffer
 	KickCommandBuffer();
+	// TODO: Remove when https://github.com/libsdl-org/SDL/issues/5203 is fixed
+#ifdef CONF_PLATFORM_MACOS
+	if(str_find(GetVersionString(), "Metal"))
+		WaitForIdle();
+#endif
 }
 
 bool CGraphics_Threaded::SetVSync(bool State)
@@ -2530,25 +2627,16 @@ int CGraphics_Threaded::GetVideoModes(CVideoMode *pModes, int MaxModes, int Scre
 	mem_zero(&Image, sizeof(Image));
 
 	int NumModes = 0;
-	CCommandBuffer::SCommand_VideoModes Cmd;
-	Cmd.m_pModes = pModes;
-	Cmd.m_MaxModes = MaxModes;
-	Cmd.m_pNumModes = &NumModes;
-	Cmd.m_HiDPIScale = m_ScreenHiDPIScale;
-	Cmd.m_MaxWindowWidth = g_Config.m_GfxDesktopWidth;
-	Cmd.m_MaxWindowHeight = g_Config.m_GfxDesktopHeight;
-	Cmd.m_Screen = Screen;
+	m_pBackend->GetVideoModes(pModes, MaxModes, &NumModes, m_ScreenHiDPIScale, g_Config.m_GfxDesktopWidth, g_Config.m_GfxDesktopHeight, Screen);
 
-	if(!AddCmd(
-		   Cmd, [] { return true; }, "failed to add video mode command"))
-	{
-		return 0;
-	}
-
-	// kick the buffer and wait for the result and return it
-	KickCommandBuffer();
-	WaitForIdle();
 	return NumModes;
 }
 
-extern IEngineGraphics *CreateEngineGraphicsThreaded() { return new CGraphics_Threaded(); }
+extern IEngineGraphics *CreateEngineGraphicsThreaded()
+{
+#ifdef CONF_HEADLESS_CLIENT
+	return new CGraphics_ThreadedNull();
+#else
+	return new CGraphics_Threaded();
+#endif
+}
