@@ -124,9 +124,11 @@ public:
 		// misc
 		CMD_VSYNC,
 		CMD_SCREENSHOT,
-		CMD_VIDEOMODES,
 		CMD_UPDATE_VIEWPORT,
 
+		// in Android a window that minimizes gets destroyed
+		CMD_WINDOW_CREATE_NTF,
+		CMD_WINDOW_DESTROY_NTF,
 	};
 
 	enum
@@ -489,20 +491,6 @@ public:
 		CImageInfo *m_pImage; // processor will fill this out, the one who adds this command must free the data as well
 	};
 
-	struct SCommand_VideoModes : public SCommand
-	{
-		SCommand_VideoModes() :
-			SCommand(CMD_VIDEOMODES) {}
-
-		CVideoMode *m_pModes; // processor will fill this in
-		int m_MaxModes; // maximum of modes the processor can write to the m_pModes
-		int *m_pNumModes; // processor will write to this pointer
-		int m_HiDPIScale;
-		int m_MaxWindowWidth;
-		int m_MaxWindowHeight;
-		int m_Screen;
-	};
-
 	struct SCommand_Swap : public SCommand
 	{
 		SCommand_Swap() :
@@ -577,6 +565,22 @@ public:
 		int m_Slot;
 	};
 
+	struct SCommand_WindowCreateNtf : public CCommandBuffer::SCommand
+	{
+		SCommand_WindowCreateNtf() :
+			SCommand(CMD_WINDOW_CREATE_NTF) {}
+
+		uint32_t m_WindowID;
+	};
+
+	struct SCommand_WindowDestroyNtf : public CCommandBuffer::SCommand
+	{
+		SCommand_WindowDestroyNtf() :
+			SCommand(CMD_WINDOW_DESTROY_NTF) {}
+
+		uint32_t m_WindowID;
+	};
+
 	//
 	CCommandBuffer(unsigned CmdBufferSize, unsigned DataBufferSize) :
 		m_CmdBuffer(CmdBufferSize), m_DataBuffer(DataBufferSize), m_pCmdBufferHead(nullptr), m_pCmdBufferTail(nullptr)
@@ -653,25 +657,33 @@ public:
 
 	virtual ~IGraphicsBackend() {}
 
-	virtual int Init(const char *pName, int *Screen, int *pWidth, int *pHeight, int FsaaSamples, int Flags, int *pDesktopWidth, int *pDesktopHeight, int *pCurrentWidth, int *pCurrentHeight, class IStorage *pStorage) = 0;
+	virtual int Init(const char *pName, int *Screen, int *pWidth, int *pHeight, int *pRefreshRate, int FsaaSamples, int Flags, int *pDesktopWidth, int *pDesktopHeight, int *pCurrentWidth, int *pCurrentHeight, class IStorage *pStorage) = 0;
 	virtual int Shutdown() = 0;
 
 	virtual int MemoryUsage() const = 0;
 
+	virtual void GetVideoModes(CVideoMode *pModes, int MaxModes, int *pNumModes, int HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int Screen) = 0;
+	virtual void GetCurrentVideoMode(CVideoMode &CurMode, int HiDPIScale, int MaxWindowWidth, int MaxWindowHeight, int Screen) = 0;
+
 	virtual int GetNumScreens() const = 0;
 
 	virtual void Minimize() = 0;
 	virtual void Maximize() = 0;
-	virtual void SetWindowParams(int FullscreenMode, bool IsBorderless) = 0;
+	virtual void SetWindowParams(int FullscreenMode, bool IsBorderless, bool AllowResizing) = 0;
 	virtual bool SetWindowScreen(int Index) = 0;
+	virtual bool UpdateDisplayMode(int Index) = 0;
 	virtual int GetWindowScreen() = 0;
 	virtual int WindowActive() = 0;
 	virtual int WindowOpen() = 0;
 	virtual void SetWindowGrab(bool Grab) = 0;
-	virtual void ResizeWindow(int w, int h) = 0;
+	// returns true, if the video mode changed
+	virtual bool ResizeWindow(int w, int h, int RefreshRate) = 0;
 	virtual void GetViewportSize(int &w, int &h) = 0;
 	virtual void NotifyWindow() = 0;
 
+	virtual void WindowDestroyNtf(uint32_t WindowID) = 0;
+	virtual void WindowCreateNtf(uint32_t WindowID) = 0;
+
 	virtual void RunBuffer(CCommandBuffer *pBuffer) = 0;
 	virtual bool IsIdle() const = 0;
 	virtual void WaitForIdle() = 0;
@@ -763,11 +775,13 @@ class CGraphics_Threaded : public IEngineGraphics
 
 	struct SQuadContainer
 	{
-		SQuadContainer()
+		SQuadContainer(bool AutomaticUpload = true)
 		{
 			m_Quads.clear();
 			m_QuadBufferObjectIndex = m_QuadBufferContainerIndex = -1;
 			m_FreeIndex = -1;
+
+			m_AutomaticUpload = AutomaticUpload;
 		}
 
 		struct SQuad
@@ -781,6 +795,8 @@ class CGraphics_Threaded : public IEngineGraphics
 		int m_QuadBufferContainerIndex;
 
 		int m_FreeIndex;
+
+		bool m_AutomaticUpload;
 	};
 	std::vector<SQuadContainer> m_QuadContainers;
 	int m_FirstFreeQuadContainer;
@@ -788,7 +804,7 @@ class CGraphics_Threaded : public IEngineGraphics
 	struct SWindowResizeListener
 	{
 		SWindowResizeListener(WINDOW_RESIZE_FUNC pFunc, void *pUser) :
-			m_pFunc(pFunc), m_pUser(pUser) {}
+			m_pFunc(std::move(pFunc)), m_pUser(pUser) {}
 		WINDOW_RESIZE_FUNC m_pFunc;
 		void *m_pUser;
 	};
@@ -867,8 +883,7 @@ public:
 	void LinesEnd() override;
 	void LinesDraw(const CLineItem *pArray, int Num) override;
 
-	int UnloadTexture(IGraphics::CTextureHandle Index) override;
-	int UnloadTextureNew(CTextureHandle &TextureHandle) override;
+	int UnloadTexture(IGraphics::CTextureHandle *pIndex) override;
 	IGraphics::CTextureHandle LoadTextureRaw(int Width, int Height, int Format, const void *pData, int StoreFormat, int Flags, const char *pTexName = NULL) override;
 	int LoadTextureRawSub(IGraphics::CTextureHandle TextureID, int x, int y, int Width, int Height, int Format, const void *pData) override;
 
@@ -1031,14 +1046,25 @@ public:
 	void QuadsDrawFreeform(const CFreeformItem *pArray, int Num) override;
 	void QuadsText(float x, float y, float Size, const char *pText) override;
 
-	int CreateQuadContainer() override;
+	const GL_STexCoord *GetCurTextureCoordinates() override
+	{
+		return m_aTexture;
+	}
+
+	const GL_SColor *GetCurColor() override
+	{
+		return m_aColor;
+	}
+
+	int CreateQuadContainer(bool AutomaticUpload = true) override;
+	void QuadContainerChangeAutomaticUpload(int ContainerIndex, bool AutomaticUpload) override;
 	void QuadContainerUpload(int ContainerIndex) override;
 	void QuadContainerAddQuads(int ContainerIndex, CQuadItem *pArray, int Num) override;
 	void QuadContainerAddQuads(int ContainerIndex, CFreeformItem *pArray, int Num) override;
 	void QuadContainerReset(int ContainerIndex) override;
 	void DeleteQuadContainer(int ContainerIndex) override;
 	void RenderQuadContainer(int ContainerIndex, int QuadDrawNum) override;
-	void RenderQuadContainer(int ContainerIndex, int QuadOffset, int QuadDrawNum) override;
+	void RenderQuadContainer(int ContainerIndex, int QuadOffset, int QuadDrawNum, bool ChangeWrapMode = true) override;
 	void RenderQuadContainerEx(int ContainerIndex, int QuadOffset, int QuadDrawNum, float X, float Y, float ScaleX = 1.f, float ScaleY = 1.f) override;
 	void RenderQuadContainerAsSprite(int ContainerIndex, int QuadOffset, float X, float Y, float ScaleX = 1.f, float ScaleY = 1.f) override;
 	void RenderQuadContainerAsSpriteMultiple(int ContainerIndex, int QuadOffset, int DrawCount, SRenderSpriteInfo *pRenderInfo) override;
@@ -1143,12 +1169,17 @@ public:
 	int GetNumScreens() const override;
 	void Minimize() override;
 	void Maximize() override;
-	void SetWindowParams(int FullscreenMode, bool IsBorderless) override;
+	void SetWindowParams(int FullscreenMode, bool IsBorderless, bool AllowResizing) override;
 	bool SetWindowScreen(int Index) override;
-	void Resize(int w, int h, bool SetWindowSize = false) override;
+	void Move(int x, int y) override;
+	void Resize(int w, int h, int RefreshRate) override;
+	void GotResized(int w, int h, int RefreshRate) override;
 	void AddWindowResizeListener(WINDOW_RESIZE_FUNC pFunc, void *pUser) override;
 	int GetWindowScreen() override;
 
+	void WindowDestroyNtf(uint32_t WindowID) override;
+	void WindowCreateNtf(uint32_t WindowID) override;
+
 	int WindowActive() override;
 	int WindowOpen() override;
 
@@ -1165,8 +1196,8 @@ public:
 
 	int GetVideoModes(CVideoMode *pModes, int MaxModes, int Screen) override;
 
-	virtual int GetDesktopScreenWidth() const { return g_Config.m_GfxScreenWidth; }
-	virtual int GetDesktopScreenHeight() const { return g_Config.m_GfxScreenHeight; }
+	virtual int GetDesktopScreenWidth() const { return g_Config.m_GfxDesktopWidth; }
+	virtual int GetDesktopScreenHeight() const { return g_Config.m_GfxDesktopHeight; }
 
 	// synchronization
 	void InsertSignal(CSemaphore *pSemaphore) override;
