@@ -86,7 +86,7 @@ void EscapeUrl(char *pBuf, int Size, const char *pStr)
 	curl_free(pEsc);
 }
 
-CRequest::CRequest(const char *pUrl, CTimeout Timeout, bool LogProgress) :
+CRequest::CRequest(const char *pUrl, CTimeout Timeout, HTTPLOG LogProgress) :
 	m_Timeout(Timeout),
 	m_Size(0),
 	m_Progress(0),
@@ -148,24 +148,34 @@ int CRequest::RunImpl(CURL *pHandle)
 	curl_easy_setopt(pHandle, CURLOPT_PROGRESSDATA, this);
 	curl_easy_setopt(pHandle, CURLOPT_PROGRESSFUNCTION, ProgressCallback);
 
+	if(curl_version_info(CURLVERSION_NOW)->version_num < 0x076800)
+	{
+		// Causes crashes, see https://github.com/ddnet/ddnet/issues/4342
+		curl_easy_setopt(pHandle, CURLOPT_FORBID_REUSE, 1L);
+	}
+
+#ifdef CONF_PLATFORM_ANDROID
+	curl_easy_setopt(pHandle, CURLOPT_CAINFO, "data/cacert.pem");
+#endif
+
 	if(!AfterInit(pHandle))
 	{
 		return HTTP_ERROR;
 	}
 
-	if(g_Config.m_DbgCurl || m_LogProgress)
-		dbg_msg("http", "http %s", m_aUrl);
+	if(g_Config.m_DbgCurl || m_LogProgress >= HTTPLOG::ALL)
+		dbg_msg("http", "fetching %s", m_aUrl);
 	m_State = HTTP_RUNNING;
 	int Ret = curl_easy_perform(pHandle);
 	if(Ret != CURLE_OK)
 	{
-		if(g_Config.m_DbgCurl || m_LogProgress)
-			dbg_msg("http", "task failed. libcurl error: %s", aErr);
+		if(g_Config.m_DbgCurl || m_LogProgress >= HTTPLOG::FAILURE)
+			dbg_msg("http", "%s failed. libcurl error: %s", m_aUrl, aErr);
 		return (Ret == CURLE_ABORTED_BY_CALLBACK) ? HTTP_ABORTED : HTTP_ERROR;
 	}
 	else
 	{
-		if(g_Config.m_DbgCurl || m_LogProgress)
+		if(g_Config.m_DbgCurl || m_LogProgress >= HTTPLOG::ALL)
 			dbg_msg("http", "task done %s", m_aUrl);
 		return HTTP_DONE;
 	}
@@ -179,14 +189,14 @@ size_t CRequest::WriteCallback(char *pData, size_t Size, size_t Number, void *pU
 int CRequest::ProgressCallback(void *pUser, double DlTotal, double DlCurr, double UlTotal, double UlCurr)
 {
 	CGetFile *pTask = (CGetFile *)pUser;
-	pTask->m_Current = DlCurr;
-	pTask->m_Size = DlTotal;
-	pTask->m_Progress = (100 * DlCurr) / (DlTotal ? DlTotal : 1);
+	pTask->m_Current.store(DlCurr, std::memory_order_relaxed);
+	pTask->m_Size.store(DlTotal, std::memory_order_relaxed);
+	pTask->m_Progress.store((100 * DlCurr) / (DlTotal ? DlTotal : 1), std::memory_order_relaxed);
 	pTask->OnProgress();
 	return pTask->m_Abort ? -1 : 0;
 }
 
-CHead::CHead(const char *pUrl, CTimeout Timeout, bool LogProgress) :
+CHead::CHead(const char *pUrl, CTimeout Timeout, HTTPLOG LogProgress) :
 	CRequest(pUrl, Timeout, LogProgress)
 {
 }
@@ -202,7 +212,7 @@ bool CHead::AfterInit(void *pCurl)
 	return true;
 }
 
-CGet::CGet(const char *pUrl, CTimeout Timeout, bool LogProgress) :
+CGet::CGet(const char *pUrl, CTimeout Timeout, HTTPLOG LogProgress) :
 	CRequest(pUrl, Timeout, LogProgress),
 	m_BufferSize(0),
 	m_BufferLength(0),
@@ -275,7 +285,7 @@ size_t CGet::OnData(char *pData, size_t DataSize)
 	return DataSize;
 }
 
-CGetFile::CGetFile(IStorage *pStorage, const char *pUrl, const char *pDest, int StorageType, CTimeout Timeout, bool LogProgress) :
+CGetFile::CGetFile(IStorage *pStorage, const char *pUrl, const char *pDest, int StorageType, CTimeout Timeout, HTTPLOG LogProgress) :
 	CRequest(pUrl, Timeout, LogProgress),
 	m_pStorage(pStorage),
 	m_File(0),
