@@ -22,7 +22,7 @@ public:
 	{
 	public:
 		NETADDR m_Addr;
-		int64 m_RequestTime;
+		int64_t m_RequestTime;
 		bool m_RequestIgnoreInfo;
 		int m_GotInfo;
 		bool m_Request64Legacy;
@@ -176,7 +176,7 @@ private:
 	CServerEntry *m_pLastReqServer;
 	int m_NumRequests;
 
-	//used instead of g_Config.br_max_requests to get more servers
+	// used instead of g_Config.br_max_requests to get more servers
 	int m_CurrentMaxRequests;
 
 	int m_NeedRefresh;
@@ -191,8 +191,7 @@ private:
 	char m_aFilterGametypeString[128];
 
 	int m_ServerlistType;
-	int64 m_BroadcastTime;
-	int m_RequestNumber;
+	int64_t m_BroadcastTime;
 	unsigned char m_aTokenSeed[16];
 
 	bool m_SortOnNextUpdate;
@@ -217,6 +216,8 @@ private:
 	void Sort();
 	int SortHash() const;
 
+	void CleanUp();
+
 	void UpdateFromHttp();
 	CServerEntry *Add(const NETADDR &Addr);
 
@@ -228,7 +229,7 @@ private:
 	static void Con_LeakIpAddress(IConsole::IResult *pResult, void *pUserData);
 
 	void SetInfo(CServerEntry *pEntry, const CServerInfo &Info);
-	void SetLatency(const NETADDR Addr, int Latency);
+	void SetLatency(NETADDR Addr, int Latency);
 
 	static void ConfigSaveCallback(IConfigManager *pConfigManager, void *pUserData);
 };
