@@ -93,8 +93,8 @@ CUpdater::CUpdater()
 	m_Percent = 0;
 	m_Lock = lock_create();
 
-	str_format(m_aClientExecTmp, sizeof(m_aClientExecTmp), CLIENT_EXEC ".%d.tmp", pid());
-	str_format(m_aServerExecTmp, sizeof(m_aServerExecTmp), SERVER_EXEC ".%d.tmp", pid());
+	IStorage::FormatTmpPath(m_aClientExecTmp, sizeof(m_aClientExecTmp), CLIENT_EXEC);
+	IStorage::FormatTmpPath(m_aServerExecTmp, sizeof(m_aServerExecTmp), SERVER_EXEC);
 }
 
 void CUpdater::Init()
@@ -102,7 +102,6 @@ void CUpdater::Init()
 	m_pClient = Kernel()->RequestInterface<IClient>();
 	m_pStorage = Kernel()->RequestInterface<IStorage>();
 	m_pEngine = Kernel()->RequestInterface<IEngine>();
-	m_IsWinXP = os_is_winxp_or_lower();
 }
 
 CUpdater::~CUpdater()
@@ -201,16 +200,13 @@ bool CUpdater::ReplaceClient()
 {
 	dbg_msg("updater", "replacing " PLAT_CLIENT_EXEC);
 	bool Success = true;
-	char aPath[512];
+	char aPath[IO_MAX_PATH_LENGTH];
 
 	// Replace running executable by renaming twice...
-	if(!m_IsWinXP)
-	{
-		m_pStorage->RemoveBinaryFile(CLIENT_EXEC ".old");
-		Success &= m_pStorage->RenameBinaryFile(PLAT_CLIENT_EXEC, CLIENT_EXEC ".old");
-		str_format(aPath, sizeof(aPath), "update/%s", m_aClientExecTmp);
-		Success &= m_pStorage->RenameBinaryFile(aPath, PLAT_CLIENT_EXEC);
-	}
+	m_pStorage->RemoveBinaryFile(CLIENT_EXEC ".old");
+	Success &= m_pStorage->RenameBinaryFile(PLAT_CLIENT_EXEC, CLIENT_EXEC ".old");
+	str_format(aPath, sizeof(aPath), "update/%s", m_aClientExecTmp);
+	Success &= m_pStorage->RenameBinaryFile(aPath, PLAT_CLIENT_EXEC);
 #if !defined(CONF_FAMILY_WINDOWS)
 	m_pStorage->GetBinaryPath(PLAT_CLIENT_EXEC, aPath, sizeof aPath);
 	char aBuf[512];
@@ -228,7 +224,7 @@ bool CUpdater::ReplaceServer()
 {
 	dbg_msg("updater", "replacing " PLAT_SERVER_EXEC);
 	bool Success = true;
-	char aPath[512];
+	char aPath[IO_MAX_PATH_LENGTH];
 
 	//Replace running executable by renaming twice...
 	m_pStorage->RemoveBinaryFile(SERVER_EXEC ".old");
@@ -250,8 +246,8 @@ bool CUpdater::ReplaceServer()
 
 void CUpdater::ParseUpdate()
 {
-	char aPath[512];
-	IOHANDLE File = m_pStorage->OpenFile(m_pStorage->GetBinaryPath("update/update.json", aPath, sizeof aPath), IOFLAG_READ, IStorage::TYPE_ABSOLUTE);
+	char aPath[IO_MAX_PATH_LENGTH];
+	IOHANDLE File = m_pStorage->OpenFile(m_pStorage->GetBinaryPath("update/update.json", aPath, sizeof aPath), IOFLAG_READ | IOFLAG_SKIP_BOM, IStorage::TYPE_ABSOLUTE);
 	if(!File)
 		return;
 
@@ -387,23 +383,6 @@ void CUpdater::CommitUpdate()
 		m_State = NEED_RESTART;
 	else
 	{
-		if(!m_IsWinXP)
-			m_pClient->Restart();
-		else
-			WinXpRestart();
+		m_pClient->Restart();
 	}
 }
-
-void CUpdater::WinXpRestart()
-{
-	char aBuf[512];
-	IOHANDLE bhFile = io_open(m_pStorage->GetBinaryPath("du.bat", aBuf, sizeof aBuf), IOFLAG_WRITE);
-	if(!bhFile)
-		return;
-	char bBuf[512];
-	str_format(bBuf, sizeof(bBuf), ":_R\r\ndel \"" PLAT_CLIENT_EXEC "\"\r\nif exist \"" PLAT_CLIENT_EXEC "\" goto _R\r\n:_T\r\nmove /y \"update\\%s\" \"" PLAT_CLIENT_EXEC "\"\r\nif not exist \"" PLAT_CLIENT_EXEC "\" goto _T\r\nstart " PLAT_CLIENT_EXEC "\r\ndel \"du.bat\"\r\n", m_aClientExecTmp);
-	io_write(bhFile, bBuf, str_length(bBuf));
-	io_close(bhFile);
-	shell_execute(aBuf);
-	m_pClient->Quit();
-}
