@@ -74,7 +74,7 @@ void CVideo::Start()
 	char aDate[20];
 	str_timestamp(aDate, sizeof(aDate));
 	char aBuf[256];
-	if(strlen(m_Name) != 0)
+	if(str_length(m_Name) != 0)
 		str_format(aBuf, sizeof(aBuf), "videos/%s", m_Name);
 	else
 		str_format(aBuf, sizeof(aBuf), "videos/%s.mp4", aDate);
@@ -230,7 +230,7 @@ void CVideo::NextVideoFrameThread()
 		if(m_Vseq >= 2)
 		{
 			m_ProcessingVideoFrame = true;
-			m_VideoStream.pFrame->pts = (int64)m_VideoStream.pEnc->frame_number;
+			m_VideoStream.pFrame->pts = (int64_t)m_VideoStream.pEnc->frame_number;
 			//dbg_msg("video_recorder", "vframe: %d", m_VideoStream.pEnc->frame_number);
 
 			ReadRGBFromGL();
@@ -416,7 +416,7 @@ AVFrame *CVideo::AllocPicture(enum AVPixelFormat PixFmt, int Width, int Height)
 	return pPicture;
 }
 
-AVFrame *CVideo::AllocAudioFrame(enum AVSampleFormat SampleFmt, uint64 ChannelLayout, int SampleRate, int NbSamples)
+AVFrame *CVideo::AllocAudioFrame(enum AVSampleFormat SampleFmt, uint64_t ChannelLayout, int SampleRate, int NbSamples)
 {
 	AVFrame *Frame = av_frame_alloc();
 	int Ret;
@@ -563,7 +563,7 @@ bool CVideo::OpenAudio()
 }
 
 /* Add an output stream. */
-bool CVideo::AddStream(OutputStream *pStream, AVFormatContext *pOC, AVCodec **ppCodec, enum AVCodecID CodecId)
+bool CVideo::AddStream(OutputStream *pStream, AVFormatContext *pOC, const AVCodec **ppCodec, enum AVCodecID CodecId)
 {
 	AVCodecContext *c;
 
