@@ -27,7 +27,7 @@ typedef struct OutputStream
 	AVCodecContext *pEnc;
 
 	/* pts of the next frame that will be generated */
-	int64 NextPts;
+	int64_t NextPts;
 	int SamplesCount;
 
 	AVFrame *pFrame;
@@ -69,13 +69,13 @@ private:
 	bool OpenVideo();
 	bool OpenAudio();
 	AVFrame *AllocPicture(enum AVPixelFormat PixFmt, int Width, int Height);
-	AVFrame *AllocAudioFrame(enum AVSampleFormat SampleFmt, uint64 ChannelLayout, int SampleRate, int NbSamples);
+	AVFrame *AllocAudioFrame(enum AVSampleFormat SampleFmt, uint64_t ChannelLayout, int SampleRate, int NbSamples);
 
 	void WriteFrame(OutputStream *pStream) REQUIRES(g_WriteLock);
 	void FinishFrames(OutputStream *pStream);
 	void CloseStream(OutputStream *pStream);
 
-	bool AddStream(OutputStream *pStream, AVFormatContext *pOC, AVCodec **ppCodec, enum AVCodecID CodecId);
+	bool AddStream(OutputStream *pStream, AVFormatContext *pOC, const AVCodec **ppCodec, enum AVCodecID CodecId);
 
 	class CGraphics_Threaded *m_pGraphics;
 	class IStorage *m_pStorage;
@@ -107,13 +107,13 @@ private:
 	OutputStream m_VideoStream;
 	OutputStream m_AudioStream;
 
-	AVCodec *m_VideoCodec;
-	AVCodec *m_AudioCodec;
+	const AVCodec *m_VideoCodec;
+	const AVCodec *m_AudioCodec;
 
 	AVDictionary *m_pOptDict;
 
 	AVFormatContext *m_pFormatContext;
-	AVOutputFormat *m_pFormat;
+	const AVOutputFormat *m_pFormat;
 
 	uint8_t *m_pRGB;
 
