@@ -32,8 +32,10 @@ public:
 	virtual void Init() = 0;
 	virtual void InitLogfile() = 0;
 	virtual void AddJob(std::shared_ptr<IJob> pJob) = 0;
+	static void RunJobBlocking(IJob *pJob);
 };
 
 extern IEngine *CreateEngine(const char *pAppname, bool Silent, int Jobs);
+extern IEngine *CreateTestEngine(const char *pAppname, int Jobs);
 
 #endif
