@@ -21,12 +21,12 @@ public:
 
 	virtual int RefreshAddresses(int Nettype) = 0;
 	virtual void Update() = 0;
-	virtual int IsRefreshing() = 0;
-	virtual NETADDR GetAddr(int Index) = 0;
+	virtual bool IsRefreshing() const = 0;
+	virtual NETADDR GetAddr(int Index) const = 0;
 	virtual void SetCount(int Index, int Count) = 0;
-	virtual int GetCount(int Index) = 0;
-	virtual const char *GetName(int Index) = 0;
-	virtual bool IsValid(int Index) = 0;
+	virtual int GetCount(int Index) const = 0;
+	virtual const char *GetName(int Index) const = 0;
+	virtual bool IsValid(int Index) const = 0;
 };
 
 class IEngineMasterServer : public IMasterServer
