@@ -33,7 +33,9 @@ public:
 	virtual void OnEngineTick();
 	virtual void OnEngineClientJoin(int ClientID, bool Sixup);
 	virtual void OnEngineClientDrop(int ClientID, const char *pReason);
-	virtual void OnEngineClientMessage(int ClientID, const void *pData, int Size, int Flags);
+	virtual bool OnEngineClientMessage(int ClientID, const void *pData, int Size, int Flags);
+	virtual bool OnEngineServerMessage(int ClientID, const void *pData, int Size, int Flags);
+	virtual bool OnEngineSimulateClientMessage(int *pClientID, void *pBuffer, int BufferSize, int *pOutSize, int *pFlags);
 
 	// Game
 	virtual void RoundStart(class IGameServer *pGameServer);
@@ -44,7 +46,7 @@ public:
 	virtual void OnSpawn(int ClientID);
 	virtual void OnHammerFireReloading(int ClientID);
 	virtual void OnHammerFire(int ClientID);
-	virtual void OnHammerHit(int ClientID);
+	virtual void OnHammerHit(int ClientID, int TargetID);
 	virtual void OnDirectInput(int ClientID);
 	virtual void OnCharacterTick(int ClientID);
 	virtual void OnHookAttach(int ClientID, bool Player);
