@@ -23,6 +23,11 @@ public:
 	virtual const char *InsertIgnore() const { return "INSERT OR IGNORE"; };
 	virtual const char *Random() const { return "RANDOM()"; };
 	virtual const char *MedianMapTime(char *pBuffer, int BufferSize) const;
+	// Since SQLite 3.23.0 true/false literals are recognized, but still cleaner to use 1/0, because:
+	// > For compatibility, if there exist columns named "true" or "false", then
+	// > the identifiers refer to the columns rather than Boolean constants.
+	virtual const char *False() const { return "0"; }
+	virtual const char *True() const { return "1"; }
 
 	virtual bool Connect(char *pError, int ErrorSize);
 	virtual void Disconnect();
@@ -32,6 +37,7 @@ public:
 	virtual void BindString(int Idx, const char *pString);
 	virtual void BindBlob(int Idx, unsigned char *pBlob, int Size);
 	virtual void BindInt(int Idx, int Value);
+	virtual void BindInt64(int Idx, int64_t Value);
 	virtual void BindFloat(int Idx, float Value);
 
 	virtual void Print();
@@ -41,6 +47,7 @@ public:
 	virtual bool IsNull(int Col);
 	virtual float GetFloat(int Col);
 	virtual int GetInt(int Col);
+	virtual int64_t GetInt64(int Col);
 	virtual void GetString(int Col, char *pBuffer, int BufferSize);
 	// passing a negative buffer size is undefined behavior
 	virtual int GetBlob(int Col, unsigned char *pBuffer, int BufferSize);
@@ -49,7 +56,7 @@ public:
 
 private:
 	// copy of config vars
-	char m_aFilename[512];
+	char m_aFilename[IO_MAX_PATH_LENGTH];
 	bool m_Setup;
 
 	sqlite3 *m_pDb;
@@ -115,6 +122,11 @@ bool CSqliteConnection::Connect(char *pError, int ErrorSize)
 		return false;
 	}
 
+	if(sqlite3_libversion_number() < 3025000)
+	{
+		dbg_msg("sql", "SQLite version %s is not supported, use at least version 3.25.0", sqlite3_libversion());
+	}
+
 	int Result = sqlite3_open(m_aFilename, &m_pDb);
 	if(Result != SQLITE_OK)
 	{
@@ -196,6 +208,13 @@ void CSqliteConnection::BindInt(int Idx, int Value)
 	m_Done = false;
 }
 
+void CSqliteConnection::BindInt64(int Idx, int64_t Value)
+{
+	int Result = sqlite3_bind_int64(m_pStmt, Idx, Value);
+	AssertNoError(Result);
+	m_Done = false;
+}
+
 void CSqliteConnection::BindFloat(int Idx, float Value)
 {
 	int Result = sqlite3_bind_double(m_pStmt, Idx, (double)Value);
@@ -205,7 +224,7 @@ void CSqliteConnection::BindFloat(int Idx, float Value)
 
 // TODO(2020-09-07): remove extern declaration, when all supported systems ship SQLite3 version 3.14 or above
 #if defined(__GNUC__)
-extern char *sqlite3_expanded_sql(sqlite3_stmt *pStmt) __attribute__((weak));
+extern char *sqlite3_expanded_sql(sqlite3_stmt *pStmt) __attribute__((weak)); // NOLINT(readability-redundant-declaration)
 #endif
 
 void CSqliteConnection::Print()
@@ -274,6 +293,11 @@ int CSqliteConnection::GetInt(int Col)
 	return sqlite3_column_int(m_pStmt, Col - 1);
 }
 
+int64_t CSqliteConnection::GetInt64(int Col)
+{
+	return sqlite3_column_int64(m_pStmt, Col - 1);
+}
+
 void CSqliteConnection::GetString(int Col, char *pBuffer, int BufferSize)
 {
 	str_copy(pBuffer, (const char *)sqlite3_column_text(m_pStmt, Col - 1), BufferSize);
@@ -343,7 +367,7 @@ bool CSqliteConnection::AddPoints(const char *pPlayer, int Points, char *pError,
 	str_format(aBuf, sizeof(aBuf),
 		"INSERT INTO %s_points(Name, Points) "
 		"VALUES (?, ?) "
-		"ON CONFLICT(Name) DO UPDATE SET Points=Points+?;",
+		"ON CONFLICT(Name) DO UPDATE SET Points=Points+?",
 		GetPrefix());
 	if(PrepareStatement(aBuf, pError, ErrorSize))
 	{
@@ -353,14 +377,10 @@ bool CSqliteConnection::AddPoints(const char *pPlayer, int Points, char *pError,
 	BindInt(2, Points);
 	BindInt(3, Points);
 	bool End;
-	if(Step(&End, pError, ErrorSize))
-	{
-		return true;
-	}
-	return false;
+	return Step(&End, pError, ErrorSize);
 }
 
-IDbConnection *CreateSqliteConnection(const char *pFilename, bool Setup)
+std::unique_ptr<IDbConnection> CreateSqliteConnection(const char *pFilename, bool Setup)
 {
-	return new CSqliteConnection(pFilename, Setup);
+	return std::unique_ptr<IDbConnection>(new CSqliteConnection(pFilename, Setup));
 }
