@@ -2,13 +2,21 @@
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
 #ifndef ENGINE_SHARED_COMPRESSION_H
 #define ENGINE_SHARED_COMPRESSION_H
+
 // variable int packing
 class CVariableInt
 {
 public:
+	enum
+	{
+		MAX_BYTES_PACKED = 5, // maximum number of bytes in a packed int
+	};
+
 	static unsigned char *Pack(unsigned char *pDst, int i);
 	static const unsigned char *Unpack(const unsigned char *pSrc, int *pInOut);
-	static long Compress(const void *pSrc, int Size, void *pDst, int DstSize);
-	static long Decompress(const void *pSrc, int Size, void *pDst, int DstSize);
+
+	static long Compress(const void *pSrc, int SrcSize, void *pDst, int DstSize);
+	static long Decompress(const void *pSrc, int SrcSize, void *pDst, int DstSize);
 };
+
 #endif
