@@ -10,6 +10,11 @@
 
 #include <zlib.h>
 
+enum
+{
+	ITEMTYPE_EX = 0xffff,
+};
+
 // raw datafile access
 class CDataFileReader
 {
@@ -35,17 +40,17 @@ public:
 	int GetDataSize(int Index);
 	void UnloadData(int Index);
 	void *GetItem(int Index, int *pType, int *pID);
-	int GetItemSize(int Index);
+	int GetItemSize(int Index) const;
 	void GetType(int Type, int *pStart, int *pNum);
 	int FindItemIndex(int Type, int ID);
 	void *FindItem(int Type, int ID);
-	int NumItems();
-	int NumData();
+	int NumItems() const;
+	int NumData() const;
 	void Unload();
 
-	SHA256_DIGEST Sha256();
-	unsigned Crc();
-	int MapSize();
+	SHA256_DIGEST Sha256() const;
+	unsigned Crc() const;
+	int MapSize() const;
 	IOHANDLE File();
 };
 
