@@ -28,7 +28,7 @@ static const unsigned char s_VersionTickCompression = 5; // demo files with this
 static const int s_LengthOffset = 152;
 static const int s_NumMarkersOffset = 176;
 
-static const ColorRGBA gs_DemoPrintColor{0.7f, 0.7f, 0.7f, 1.0f};
+static const ColorRGBA gs_DemoPrintColor{0.75f, 0.7f, 0.7f, 1.0f};
 
 CDemoRecorder::CDemoRecorder(class CSnapshotDelta *pSnapshotDelta, bool NoMapData)
 {
@@ -42,7 +42,7 @@ CDemoRecorder::CDemoRecorder(class CSnapshotDelta *pSnapshotDelta, bool NoMapDat
 }
 
 // Record
-int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, const char *pFilename, const char *pNetVersion, const char *pMap, SHA256_DIGEST *pSha256, unsigned Crc, const char *pType, unsigned int MapSize, unsigned char *pMapData, IOHANDLE MapFile, DEMOFUNC_FILTER pfnFilter, void *pUser)
+int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, const char *pFilename, const char *pNetVersion, const char *pMap, SHA256_DIGEST *pSha256, unsigned Crc, const char *pType, unsigned MapSize, unsigned char *pMapData, IOHANDLE MapFile, DEMOFUNC_FILTER pfnFilter, void *pUser)
 {
 	m_pfnFilter = pfnFilter;
 	m_pUser = pUser;
@@ -130,14 +130,8 @@ int CDemoRecorder::Start(class IStorage *pStorage, class IConsole *pConsole, con
 	Header.m_Version = s_CurVersion;
 	str_copy(Header.m_aNetversion, pNetVersion, sizeof(Header.m_aNetversion));
 	str_copy(Header.m_aMapName, pMap, sizeof(Header.m_aMapName));
-	Header.m_aMapSize[0] = (MapSize >> 24) & 0xff;
-	Header.m_aMapSize[1] = (MapSize >> 16) & 0xff;
-	Header.m_aMapSize[2] = (MapSize >> 8) & 0xff;
-	Header.m_aMapSize[3] = (MapSize)&0xff;
-	Header.m_aMapCrc[0] = (Crc >> 24) & 0xff;
-	Header.m_aMapCrc[1] = (Crc >> 16) & 0xff;
-	Header.m_aMapCrc[2] = (Crc >> 8) & 0xff;
-	Header.m_aMapCrc[3] = (Crc)&0xff;
+	uint_to_bytes_be(Header.m_aMapSize, MapSize);
+	uint_to_bytes_be(Header.m_aMapCrc, Crc);
 	str_copy(Header.m_aType, pType, sizeof(Header.m_aType));
 	// Header.m_Length - add this on stop
 	str_timestamp(Header.m_aTimestamp, sizeof(Header.m_aTimestamp));
@@ -225,10 +219,7 @@ void CDemoRecorder::WriteTickMarker(int Tick, int Keyframe)
 	{
 		unsigned char aChunk[5];
 		aChunk[0] = CHUNKTYPEFLAG_TICKMARKER;
-		aChunk[1] = (Tick >> 24) & 0xff;
-		aChunk[2] = (Tick >> 16) & 0xff;
-		aChunk[3] = (Tick >> 8) & 0xff;
-		aChunk[4] = (Tick)&0xff;
+		uint_to_bytes_be(aChunk + 1, Tick);
 
 		if(Keyframe)
 			aChunk[0] |= CHUNKTICKFLAG_KEYFRAME;
@@ -349,30 +340,19 @@ int CDemoRecorder::Stop()
 
 	// add the demo length to the header
 	io_seek(m_File, s_LengthOffset, IOSEEK_START);
-	int DemoLength = Length();
-	char aLength[4];
-	aLength[0] = (DemoLength >> 24) & 0xff;
-	aLength[1] = (DemoLength >> 16) & 0xff;
-	aLength[2] = (DemoLength >> 8) & 0xff;
-	aLength[3] = (DemoLength)&0xff;
+	unsigned char aLength[4];
+	int_to_bytes_be(aLength, Length());
 	io_write(m_File, aLength, sizeof(aLength));
 
 	// add the timeline markers to the header
 	io_seek(m_File, s_NumMarkersOffset, IOSEEK_START);
-	char aNumMarkers[4];
-	aNumMarkers[0] = (m_NumTimelineMarkers >> 24) & 0xff;
-	aNumMarkers[1] = (m_NumTimelineMarkers >> 16) & 0xff;
-	aNumMarkers[2] = (m_NumTimelineMarkers >> 8) & 0xff;
-	aNumMarkers[3] = (m_NumTimelineMarkers)&0xff;
+	unsigned char aNumMarkers[4];
+	int_to_bytes_be(aNumMarkers, m_NumTimelineMarkers);
 	io_write(m_File, aNumMarkers, sizeof(aNumMarkers));
 	for(int i = 0; i < m_NumTimelineMarkers; i++)
 	{
-		int Marker = m_aTimelineMarkers[i];
-		char aMarker[4];
-		aMarker[0] = (Marker >> 24) & 0xff;
-		aMarker[1] = (Marker >> 16) & 0xff;
-		aMarker[2] = (Marker >> 8) & 0xff;
-		aMarker[3] = (Marker)&0xff;
+		unsigned char aMarker[4];
+		int_to_bytes_be(aMarker, m_aTimelineMarkers[i]);
 		io_write(m_File, aMarker, sizeof(aMarker));
 	}
 
@@ -403,7 +383,19 @@ void CDemoRecorder::AddDemoMarker()
 		m_pConsole->Print(IConsole::OUTPUT_LEVEL_STANDARD, "demo_recorder", "Added timeline marker", gs_DemoPrintColor);
 }
 
+CDemoPlayer::CDemoPlayer(class CSnapshotDelta *pSnapshotDelta, TUpdateIntraTimesFunc &&UpdateIntraTimesFunc)
+{
+	Construct(pSnapshotDelta);
+
+	m_UpdateIntraTimesFunc = UpdateIntraTimesFunc;
+}
+
 CDemoPlayer::CDemoPlayer(class CSnapshotDelta *pSnapshotDelta)
+{
+	Construct(pSnapshotDelta);
+}
+
+void CDemoPlayer::Construct(class CSnapshotDelta *pSnapshotDelta)
 {
 	m_File = 0;
 	m_pKeyFrames = 0;
@@ -454,7 +446,7 @@ int CDemoPlayer::ReadChunkHeader(int *pType, int *pSize, int *pTick)
 			unsigned char aTickdata[4];
 			if(io_read(m_File, aTickdata, sizeof(aTickdata)) != sizeof(aTickdata))
 				return -1;
-			*pTick = (aTickdata[0] << 24) | (aTickdata[1] << 16) | (aTickdata[2] << 8) | aTickdata[3];
+			*pTick = bytes_be_to_int(aTickdata);
 		}
 	}
 	else
@@ -553,6 +545,15 @@ void CDemoPlayer::DoTick()
 	m_Info.m_Info.m_CurrentTick = m_Info.m_NextTick;
 	ChunkTick = m_Info.m_Info.m_CurrentTick;
 
+	int64_t Freq = time_freq();
+	int64_t CurtickStart = (m_Info.m_Info.m_CurrentTick) * Freq / SERVER_TICK_SPEED;
+	int64_t PrevtickStart = (m_Info.m_PreviousTick) * Freq / SERVER_TICK_SPEED;
+	m_Info.m_IntraTick = (m_Info.m_CurrentTime - PrevtickStart) / (float)(CurtickStart - PrevtickStart);
+	m_Info.m_IntraTickSincePrev = (m_Info.m_CurrentTime - PrevtickStart) / (float)(Freq / SERVER_TICK_SPEED);
+	m_Info.m_TickTime = (m_Info.m_CurrentTime - PrevtickStart) / (float)Freq;
+	if(m_UpdateIntraTimesFunc)
+		m_UpdateIntraTimesFunc();
+
 	while(1)
 	{
 		if(ReadChunkHeader(&ChunkType, &ChunkSize, &ChunkTick))
@@ -779,11 +780,11 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 		m_DemoType = DEMOTYPE_INVALID;
 
 	// read map
-	unsigned MapSize = (m_Info.m_Header.m_aMapSize[0] << 24) | (m_Info.m_Header.m_aMapSize[1] << 16) | (m_Info.m_Header.m_aMapSize[2] << 8) | (m_Info.m_Header.m_aMapSize[3]);
+	unsigned MapSize = bytes_be_to_uint(m_Info.m_Header.m_aMapSize);
 
 	// check if we already have the map
 	// TODO: improve map checking (maps folder, check crc)
-	unsigned Crc = (m_Info.m_Header.m_aMapCrc[0] << 24) | (m_Info.m_Header.m_aMapCrc[1] << 16) | (m_Info.m_Header.m_aMapCrc[2] << 8) | (m_Info.m_Header.m_aMapCrc[3]);
+	unsigned Crc = bytes_be_to_uint(m_Info.m_Header.m_aMapCrc);
 
 	// save byte offset of map for later use
 	m_MapOffset = io_tell(m_File);
@@ -798,14 +799,11 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 	if(m_Info.m_Header.m_Version > s_OldVersion)
 	{
 		// get timeline markers
-		int Num = ((m_Info.m_TimelineMarkers.m_aNumTimelineMarkers[0] << 24) & 0xFF000000) | ((m_Info.m_TimelineMarkers.m_aNumTimelineMarkers[1] << 16) & 0xFF0000) |
-			  ((m_Info.m_TimelineMarkers.m_aNumTimelineMarkers[2] << 8) & 0xFF00) | (m_Info.m_TimelineMarkers.m_aNumTimelineMarkers[3] & 0xFF);
+		int Num = bytes_be_to_int(m_Info.m_TimelineMarkers.m_aNumTimelineMarkers);
 		m_Info.m_Info.m_NumTimelineMarkers = minimum(Num, (int)MAX_TIMELINE_MARKERS);
 		for(int i = 0; i < m_Info.m_Info.m_NumTimelineMarkers; i++)
 		{
-			char *pTimelineMarker = m_Info.m_TimelineMarkers.m_aTimelineMarkers[i];
-			m_Info.m_Info.m_aTimelineMarkers[i] = ((pTimelineMarker[0] << 24) & 0xFF000000) | ((pTimelineMarker[1] << 16) & 0xFF0000) |
-							      ((pTimelineMarker[2] << 8) & 0xFF00) | (pTimelineMarker[3] & 0xFF);
+			m_Info.m_Info.m_aTimelineMarkers[i] = bytes_be_to_int(m_Info.m_TimelineMarkers.m_aTimelineMarkers[i]);
 		}
 	}
 
@@ -820,10 +818,10 @@ int CDemoPlayer::Load(class IStorage *pStorage, class IConsole *pConsole, const
 	return 0;
 }
 
-bool CDemoPlayer::ExtractMap(class IStorage *pStorage)
+unsigned char *CDemoPlayer::GetMapData(class IStorage *pStorage)
 {
 	if(!m_MapInfo.m_Size)
-		return false;
+		return 0;
 
 	long CurSeek = io_tell(m_File);
 
@@ -832,6 +830,14 @@ bool CDemoPlayer::ExtractMap(class IStorage *pStorage)
 	unsigned char *pMapData = (unsigned char *)malloc(m_MapInfo.m_Size);
 	io_read(m_File, pMapData, m_MapInfo.m_Size);
 	io_seek(m_File, CurSeek, IOSEEK_START);
+	return pMapData;
+}
+
+bool CDemoPlayer::ExtractMap(class IStorage *pStorage)
+{
+	unsigned char *pMapData = GetMapData(pStorage);
+	if(!pMapData)
+		return false;
 
 	// handle sha256
 	SHA256_DIGEST Sha256 = SHA256_ZEROED;
@@ -868,7 +874,7 @@ int CDemoPlayer::NextFrame()
 	return IsPlaying();
 }
 
-int64 CDemoPlayer::time()
+int64_t CDemoPlayer::time()
 {
 #if defined(CONF_VIDEORECORDER)
 	static bool s_Recording = false;
@@ -883,7 +889,7 @@ int64 CDemoPlayer::time()
 	}
 	else
 	{
-		int64 Now = time_get();
+		int64_t Now = time_get();
 		if(s_Recording)
 		{
 			s_Recording = false;
@@ -970,8 +976,8 @@ void CDemoPlayer::SetSpeedIndex(int Offset)
 
 int CDemoPlayer::Update(bool RealTime)
 {
-	int64 Now = time();
-	int64 Deltatime = Now - m_Info.m_LastUpdate;
+	int64_t Now = time();
+	int64_t Deltatime = Now - m_Info.m_LastUpdate;
 	m_Info.m_LastUpdate = Now;
 
 	if(!IsPlaying())
@@ -982,12 +988,12 @@ int CDemoPlayer::Update(bool RealTime)
 	}
 	else
 	{
-		int64 Freq = time_freq();
-		m_Info.m_CurrentTime += (int64)(Deltatime * (double)m_Info.m_Info.m_Speed);
+		int64_t Freq = time_freq();
+		m_Info.m_CurrentTime += (int64_t)(Deltatime * (double)m_Info.m_Info.m_Speed);
 
 		while(1)
 		{
-			int64 CurtickStart = (m_Info.m_Info.m_CurrentTick) * Freq / SERVER_TICK_SPEED;
+			int64_t CurtickStart = (m_Info.m_Info.m_CurrentTick) * Freq / SERVER_TICK_SPEED;
 
 			// break if we are ready
 			if(RealTime && CurtickStart > m_Info.m_CurrentTime)
@@ -1002,10 +1008,13 @@ int CDemoPlayer::Update(bool RealTime)
 
 		// update intratick
 		{
-			int64 CurtickStart = (m_Info.m_Info.m_CurrentTick) * Freq / SERVER_TICK_SPEED;
-			int64 PrevtickStart = (m_Info.m_PreviousTick) * Freq / SERVER_TICK_SPEED;
+			int64_t CurtickStart = (m_Info.m_Info.m_CurrentTick) * Freq / SERVER_TICK_SPEED;
+			int64_t PrevtickStart = (m_Info.m_PreviousTick) * Freq / SERVER_TICK_SPEED;
 			m_Info.m_IntraTick = (m_Info.m_CurrentTime - PrevtickStart) / (float)(CurtickStart - PrevtickStart);
+			m_Info.m_IntraTickSincePrev = (m_Info.m_CurrentTime - PrevtickStart) / (float)(Freq / SERVER_TICK_SPEED);
 			m_Info.m_TickTime = (m_Info.m_CurrentTime - PrevtickStart) / (float)Freq;
+			if(m_UpdateIntraTimesFunc)
+				m_UpdateIntraTimesFunc();
 		}
 
 		if(m_Info.m_Info.m_CurrentTick == m_Info.m_PreviousTick ||
@@ -1019,10 +1028,8 @@ int CDemoPlayer::Update(bool RealTime)
 				m_pConsole->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "demo_player", aBuf);
 			}
 		}
-
 		m_Time += m_TickTime;
 	}
-
 	return 0;
 }
 
@@ -1079,7 +1086,7 @@ bool CDemoPlayer::GetDemoInfo(class IStorage *pStorage, const char *pFilename, i
 	io_read(File, pTimelineMarkers, sizeof(CTimelineMarkers));
 
 	str_copy(pMapInfo->m_aName, pDemoHeader->m_aMapName, sizeof(pMapInfo->m_aName));
-	pMapInfo->m_Crc = (pDemoHeader->m_aMapCrc[0] << 24) | (pDemoHeader->m_aMapCrc[1] << 16) | (pDemoHeader->m_aMapCrc[2] << 8) | (pDemoHeader->m_aMapCrc[3]);
+	pMapInfo->m_Crc = bytes_be_to_int(pDemoHeader->m_aMapCrc);
 
 	SHA256_DIGEST Sha256 = SHA256_ZEROED;
 	if(pDemoHeader->m_Version >= s_Sha256Version)
@@ -1100,7 +1107,7 @@ bool CDemoPlayer::GetDemoInfo(class IStorage *pStorage, const char *pFilename, i
 	}
 	pMapInfo->m_Sha256 = Sha256;
 
-	pMapInfo->m_Size = (pDemoHeader->m_aMapSize[0] << 24) | (pDemoHeader->m_aMapSize[1] << 16) | (pDemoHeader->m_aMapSize[2] << 8) | (pDemoHeader->m_aMapSize[3]);
+	pMapInfo->m_Size = bytes_be_to_int(pDemoHeader->m_aMapSize);
 
 	io_close(File);
 	return !(mem_comp(pDemoHeader->m_aMarker, s_aHeaderMarker, sizeof(s_aHeaderMarker)) || pDemoHeader->m_Version < s_OldVersion);
@@ -1148,7 +1155,11 @@ void CDemoEditor::Slice(const char *pDemo, const char *pDst, int StartTick, int
 			Sha256 = pMapInfo->m_Sha256;
 	}
 
-	if(m_pDemoRecorder->Start(m_pStorage, m_pConsole, pDst, m_pNetVersion, pMapInfo->m_aName, &Sha256, pMapInfo->m_Crc, "client", pMapInfo->m_Size, NULL, NULL, pfnFilter, pUser) == -1)
+	unsigned char *pMapData = m_pDemoPlayer->GetMapData(m_pStorage);
+	const int Result = m_pDemoRecorder->Start(m_pStorage, m_pConsole, pDst, m_pNetVersion, pMapInfo->m_aName, &Sha256, pMapInfo->m_Crc, "client", pMapInfo->m_Size, pMapData, NULL, pfnFilter, pUser) == -1;
+	if(pMapData)
+		free(pMapData);
+	if(Result != 0)
 		return;
 
 	m_pDemoPlayer->Play();
