@@ -36,7 +36,7 @@ int CEcon::DelClientCallback(int ClientID, const char *pReason, void *pUser)
 	return 0;
 }
 
-void CEcon::SendLineCB(const char *pLine, void *pUserData, bool Highlighted)
+void CEcon::SendLineCB(const char *pLine, void *pUserData, ColorRGBA PrintColor)
 {
 	static_cast<CEcon *>(pUserData)->Send(-1, pLine);
 }
