@@ -10,7 +10,7 @@ void DilateImage(unsigned char *pImageBuff, int w, int h, int BPP);
 void DilateImageSub(unsigned char *pImageBuff, int w, int h, int BPP, int x, int y, int sw, int sh);
 
 // returned pointer is allocated with malloc
-uint8_t *ResizeImage(const uint8_t *pImgData, int Width, int Height, int NewWidth, int NewHeight, int BPP);
+uint8_t *ResizeImage(const uint8_t *pImageData, int Width, int Height, int NewWidth, int NewHeight, int BPP);
 
 int HighestBit(int OfVar);
 
