@@ -143,9 +143,9 @@ int CNetClient::Flush()
 	return m_Connection.Flush();
 }
 
-int CNetClient::GotProblems() const
+int CNetClient::GotProblems(int64_t MaxLatency) const
 {
-	if(time_get() - m_Connection.LastRecvTime() > time_freq())
+	if(time_get() - m_Connection.LastRecvTime() > MaxLatency)
 		return 1;
 	return 0;
 }
