@@ -29,3 +29,6 @@ UUID(NETMSG_CAPABILITIES, "capabilities@ddnet.tw")
 UUID(NETMSG_CLIENTVER, "clientver@ddnet.tw")
 UUID(NETMSG_PINGEX, "ping@ddnet.tw")
 UUID(NETMSG_PONGEX, "pong@ddnet.tw")
+UUID(NETMSG_CHECKSUM_REQUEST, "checksum-request@ddnet.tw")
+UUID(NETMSG_CHECKSUM_RESPONSE, "checksum-response@ddnet.tw")
+UUID(NETMSG_CHECKSUM_ERROR, "checksum-error@ddnet.tw")
