@@ -2,6 +2,7 @@
 #define ENGINE_SHARED_UUID_MANAGER_H
 
 #include <base/tl/array.h>
+#include <base/tl/sorted_array.h>
 
 enum
 {
@@ -17,15 +18,17 @@ struct CUuid
 {
 	unsigned char m_aData[16];
 
-	bool operator==(const CUuid &Other);
-	bool operator!=(const CUuid &Other);
+	bool operator==(const CUuid &Other) const;
+	bool operator!=(const CUuid &Other) const;
+	bool operator<(const CUuid &Other) const { return mem_comp(m_aData, Other.m_aData, sizeof(m_aData)) < 0; }
 };
 
 CUuid RandomUuid();
 CUuid CalculateUuid(const char *pName);
 // The buffer length should be at least UUID_MAXSTRSIZE.
 void FormatUuid(CUuid Uuid, char *pBuffer, unsigned BufferLength);
-void ParseUuid(CUuid *pUuid, char *pBuffer);
+// Returns nonzero on failure.
+int ParseUuid(CUuid *pUuid, const char *pBuffer);
 
 struct CName
 {
@@ -33,12 +36,23 @@ struct CName
 	const char *m_pName;
 };
 
+struct CNameIndexed
+{
+	CUuid m_Uuid;
+	int m_ID;
+
+	bool operator<(const CNameIndexed &Other) const { return m_Uuid < Other.m_Uuid; }
+	bool operator<(const CUuid &Other) const { return m_Uuid < Other; }
+	bool operator==(const CUuid &Other) const { return m_Uuid == Other; }
+};
+
 class CPacker;
 class CUnpacker;
 
 class CUuidManager
 {
 	array<CName> m_aNames;
+	sorted_array<CNameIndexed> m_aNamesSorted;
 
 public:
 	void RegisterName(int ID, const char *pName);
