@@ -23,17 +23,17 @@ public:
 
 	static IVideo *Current() { return ms_pCurrentVideo; }
 
-	static int64 Time() { return ms_Time; }
+	static int64_t Time() { return ms_Time; }
 	static float LocalTime() { return ms_LocalTime; }
-	static void SetLocalStartTime(int64 LocalStartTime) { ms_LocalStartTime = LocalStartTime; }
+	static void SetLocalStartTime(int64_t LocalStartTime) { ms_LocalStartTime = LocalStartTime; }
 	static void SetFPS(int FPS) { ms_TickTime = time_freq() / FPS; }
 
 protected:
 	static IVideo *ms_pCurrentVideo;
-	static int64 ms_Time;
-	static int64 ms_LocalStartTime;
+	static int64_t ms_Time;
+	static int64_t ms_LocalStartTime;
 	static float ms_LocalTime;
-	static int64 ms_TickTime;
+	static int64_t ms_TickTime;
 };
 
 #endif
