@@ -5,6 +5,10 @@
 #include <engine/keys.h>
 #include <game/client/component.h>
 
+#include "console.h"
+
+class IConfigManager;
+
 class CBinds : public CComponent
 {
 	int GetKeyID(const char *pKeyName);
@@ -20,12 +24,14 @@ class CBinds : public CComponent
 public:
 	CBinds();
 	~CBinds();
+	virtual int Sizeof() const override { return sizeof(*this); }
 
 	class CBindsSpecial : public CComponent
 	{
 	public:
 		CBinds *m_pBinds;
-		virtual bool OnInput(IInput::CEvent Event);
+		virtual int Sizeof() const override { return sizeof(*this); }
+		virtual bool OnInput(IInput::CEvent Event) override;
 	};
 
 	enum
@@ -34,6 +40,7 @@ public:
 		MODIFIER_SHIFT,
 		MODIFIER_CTRL,
 		MODIFIER_ALT,
+		MODIFIER_GUI,
 		MODIFIER_COUNT,
 		MODIFIER_COMBINATION_COUNT = 1 << MODIFIER_COUNT
 	};
@@ -52,8 +59,8 @@ public:
 	static const char *GetModifierName(int Modifier);
 	static const char *GetKeyBindModifiersName(int Modifier);
 
-	virtual void OnConsoleInit();
-	virtual bool OnInput(IInput::CEvent Event);
+	virtual void OnConsoleInit() override;
+	virtual bool OnInput(IInput::CEvent Event) override;
 
 	// DDRace
 
