@@ -12,9 +12,10 @@ class CBroadcast : public CComponent
 	float m_BroadcastRenderOffset;
 
 public:
-	virtual void OnReset();
-	virtual void OnRender();
-	virtual void OnMessage(int MsgType, void *pRawMsg);
+	virtual int Sizeof() const override { return sizeof(*this); }
+	virtual void OnReset() override;
+	virtual void OnRender() override;
+	virtual void OnMessage(int MsgType, void *pRawMsg) override;
 };
 
 #endif
