@@ -21,7 +21,6 @@ class CCamera : public CComponent
 	vec2 m_LastPos[NUM_DUMMIES];
 	vec2 m_PrevCenter;
 
-	bool m_Zooming;
 	CCubicBezier m_ZoomSmoothing;
 	float m_ZoomSmoothingStart;
 	float m_ZoomSmoothingEnd;
@@ -36,21 +35,23 @@ class CCamera : public CComponent
 public:
 	vec2 m_Center;
 	bool m_ZoomSet;
+	bool m_Zooming;
 	float m_Zoom;
 	float m_ZoomSmoothingTarget;
 
 	CCamera();
-	virtual void OnRender();
+	virtual int Sizeof() const override { return sizeof(*this); }
+	virtual void OnRender() override;
 
 	// DDRace
 
-	virtual void OnConsoleInit();
-	virtual void OnReset();
+	virtual void OnConsoleInit() override;
+	virtual void OnReset() override;
 
 private:
 	static void ConZoomPlus(IConsole::IResult *pResult, void *pUserData);
 	static void ConZoomMinus(IConsole::IResult *pResult, void *pUserData);
-	static void ConZoomReset(IConsole::IResult *pResult, void *pUserData);
+	static void ConZoom(IConsole::IResult *pResult, void *pUserData);
 	static void ConSetView(IConsole::IResult *pResult, void *pUserData);
 
 	vec2 m_ForceFreeviewPos;
