@@ -7,6 +7,8 @@
 #include <game/client/component.h>
 #include <game/client/lineinput.h>
 
+#include <game/client/skin.h>
+
 class CChat : public CComponent
 {
 	CLineInput m_Input;
@@ -22,7 +24,7 @@ class CChat : public CComponent
 
 	struct CLine
 	{
-		int64 m_Time;
+		int64_t m_Time;
 		float m_YOffset[2];
 		int m_ClientID;
 		bool m_Team;
@@ -73,6 +75,7 @@ class CChat : public CComponent
 	bool m_InputUpdate;
 	int m_ChatStringOffset;
 	int m_OldChatStringLength;
+	bool m_CompletionUsed;
 	int m_CompletionChosen;
 	char m_aCompletionBuffer[256];
 	int m_PlaceholderOffset;
@@ -99,8 +102,8 @@ class CChat : public CComponent
 	CHistoryEntry *m_pHistoryEntry;
 	CStaticRingBuffer<CHistoryEntry, 64 * 1024, CRingBufferBase::FLAG_RECYCLE> m_History;
 	int m_PendingChatCounter;
-	int64 m_LastChatSend;
-	int64 m_aLastSoundPlayed[CHAT_NUM];
+	int64_t m_LastChatSend;
+	int64_t m_aLastSoundPlayed[CHAT_NUM];
 
 	static void ConSay(IConsole::IResult *pResult, void *pUserData);
 	static void ConSayTeam(IConsole::IResult *pResult, void *pUserData);
@@ -116,6 +119,7 @@ class CChat : public CComponent
 
 public:
 	CChat();
+	virtual int Sizeof() const override { return sizeof(*this); }
 
 	static constexpr float MESSAGE_PADDING_X = 5.0f;
 	static constexpr float MESSAGE_TEE_SIZE = 7.0f;
@@ -134,15 +138,15 @@ public:
 	void RegisterCommand(const char *pName, const char *pParams, int flags, const char *pHelp);
 	void Echo(const char *pString);
 
-	virtual void OnWindowResize();
-	virtual void OnConsoleInit();
-	virtual void OnStateChange(int NewState, int OldState);
-	virtual void OnRender();
+	virtual void OnWindowResize() override;
+	virtual void OnConsoleInit() override;
+	virtual void OnStateChange(int NewState, int OldState) override;
+	virtual void OnRender() override;
 	virtual void RefindSkins();
 	virtual void OnPrepareLines();
-	virtual void OnRelease();
-	virtual void OnMessage(int MsgType, void *pRawMsg);
-	virtual bool OnInput(IInput::CEvent Event);
+	virtual void OnRelease() override;
+	virtual void OnMessage(int MsgType, void *pRawMsg) override;
+	virtual bool OnInput(IInput::CEvent Event) override;
 
 	void RebuildChat();
 };
