@@ -7,6 +7,8 @@
 #include <base/vmath.h>
 #include <game/client/component.h>
 
+#include <game/generated/protocol.h>
+
 class CControls : public CComponent
 {
 public:
@@ -17,7 +19,7 @@ public:
 	SDL_Joystick *m_Joystick;
 	bool m_JoystickFirePressed;
 	bool m_JoystickRunPressed;
-	int64 m_JoystickTapTime;
+	int64_t m_JoystickTapTime;
 
 	SDL_Joystick *m_Gamepad;
 	bool m_UsingGamepad;
@@ -33,13 +35,14 @@ public:
 	int m_OtherFire;
 
 	CControls();
+	virtual int Sizeof() const override { return sizeof(*this); }
 
-	virtual void OnReset();
-	virtual void OnRelease();
-	virtual void OnRender();
-	virtual void OnMessage(int MsgType, void *pRawMsg);
-	virtual bool OnMouseMove(float x, float y);
-	virtual void OnConsoleInit();
+	virtual void OnReset() override;
+	virtual void OnRelease() override;
+	virtual void OnRender() override;
+	virtual void OnMessage(int MsgType, void *pRawMsg) override;
+	virtual bool OnMouseMove(float x, float y) override;
+	virtual void OnConsoleInit() override;
 	virtual void OnPlayerDeath();
 
 	int SnapInput(int *pData);
