@@ -18,7 +18,8 @@ public:
 		bool operator<(const CCountryFlag &Other) const { return str_comp(m_aCountryCodeString, Other.m_aCountryCodeString) < 0; }
 	};
 
-	void OnInit();
+	virtual int Sizeof() const override { return sizeof(*this); }
+	void OnInit() override;
 
 	int Num() const;
 	const CCountryFlag *GetByCountryCode(int CountryCode) const;
@@ -35,6 +36,8 @@ private:
 	sorted_array<CCountryFlag> m_aCountryFlags;
 	int m_CodeIndexLUT[CODE_RANGE];
 
+	int m_FlagsQuadContainerIndex;
+
 	void LoadCountryflagsIndexfile();
 };
 #endif
