@@ -8,14 +8,6 @@ class CProjectileData;
 
 class CItems : public CComponent
 {
-	enum
-	{
-		MAX_EXTRA_PROJECTILES = 32,
-	};
-
-	CNetObj_Projectile m_aExtraProjectiles[MAX_EXTRA_PROJECTILES];
-	int m_NumExtraProjectiles;
-
 	void RenderProjectile(const CProjectileData *pCurrent, int ItemID);
 	void RenderPickup(const CNetObj_Pickup *pPrev, const CNetObj_Pickup *pCurrent, bool IsPredicted = false);
 	void RenderFlag(const CNetObj_Flag *pPrev, const CNetObj_Flag *pCurrent, const CNetObj_GameData *pPrevGameData, const CNetObj_GameData *pCurGameData);
@@ -24,11 +16,9 @@ class CItems : public CComponent
 	int m_ItemsQuadContainerIndex;
 
 public:
-	virtual void OnReset();
-	virtual void OnRender();
-	virtual void OnInit();
-
-	void AddExtraProjectile(CNetObj_Projectile *pProj);
+	virtual int Sizeof() const override { return sizeof(*this); }
+	virtual void OnRender() override;
+	virtual void OnInit() override;
 
 	void ReconstructSmokeTrail(const CProjectileData *pCurrent, int DestroyTick);
 };
