@@ -32,12 +32,12 @@ CMapLayers::CMapLayers(int t, bool OnlineOnly)
 void CMapLayers::OnInit()
 {
 	m_pLayers = Layers();
-	m_pImages = m_pClient->m_pMapimages;
+	m_pImages = &m_pClient->m_MapImages;
 }
 
 CCamera *CMapLayers::GetCurCamera()
 {
-	return m_pClient->m_pCamera;
+	return &m_pClient->m_Camera;
 }
 
 void CMapLayers::EnvelopeUpdate()
@@ -84,10 +84,10 @@ void CMapLayers::EnvelopeEval(int TimeOffsetMillis, int Env, float *pChannels, v
 
 	CMapItemEnvelope *pItem = (CMapItemEnvelope *)pThis->m_pLayers->Map()->GetItem(Start + Env, 0, 0);
 
-	const int64 TickToMicroSeconds = (1000000ll / (int64)pThis->Client()->GameTickSpeed());
+	const int64_t TickToMicroSeconds = (1000000ll / (int64_t)pThis->Client()->GameTickSpeed());
 
-	static int64 s_Time = 0;
-	static int64 s_LastLocalTime = time_get_microseconds();
+	static int64_t s_Time = 0;
+	static int64_t s_LastLocalTime = time_get_microseconds();
 	if(pThis->Client()->State() == IClient::STATE_DEMOPLAYBACK)
 	{
 		const IDemoPlayer::CInfo *pInfo = pThis->DemoPlayer()->BaseInfo();
@@ -104,24 +104,24 @@ void CMapLayers::EnvelopeEval(int TimeOffsetMillis, int Env, float *pChannels, v
 				// get the lerp of the current tick and prev
 				int MinTick = pThis->Client()->PrevGameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick;
 				int CurTick = pThis->Client()->GameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick;
-				s_Time = (int64)(mix<double>(
-							 0,
-							 (CurTick - MinTick),
-							 pThis->Client()->IntraGameTick(g_Config.m_ClDummy)) *
-						 TickToMicroSeconds) +
+				s_Time = (int64_t)(mix<double>(
+							   0,
+							   (CurTick - MinTick),
+							   pThis->Client()->IntraGameTick(g_Config.m_ClDummy)) *
+						   TickToMicroSeconds) +
 					 MinTick * TickToMicroSeconds;
 			}
 			else
 			{
 				int MinTick = pThis->m_LastLocalTick;
-				s_Time = (int64)(mix<double>(0,
-							 pThis->m_CurrentLocalTick - MinTick,
-							 pThis->Client()->IntraGameTick(g_Config.m_ClDummy)) *
-						 TickToMicroSeconds) +
+				s_Time = (int64_t)(mix<double>(0,
+							   pThis->m_CurrentLocalTick - MinTick,
+							   pThis->Client()->IntraGameTick(g_Config.m_ClDummy)) *
+						   TickToMicroSeconds) +
 					 MinTick * TickToMicroSeconds;
 			}
 		}
-		pThis->RenderTools()->RenderEvalEnvelope(pPoints + pItem->m_StartPoint, pItem->m_NumPoints, 4, s_Time + (int64)TimeOffsetMillis * 1000ll, pChannels);
+		CRenderTools::RenderEvalEnvelope(pPoints + pItem->m_StartPoint, pItem->m_NumPoints, 4, s_Time + (int64_t)TimeOffsetMillis * 1000ll, pChannels);
 	}
 	else
 	{
@@ -132,21 +132,21 @@ void CMapLayers::EnvelopeEval(int TimeOffsetMillis, int Env, float *pChannels, v
 				// get the lerp of the current tick and prev
 				int MinTick = pThis->Client()->PrevGameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick;
 				int CurTick = pThis->Client()->GameTick(g_Config.m_ClDummy) - pThis->m_pClient->m_Snap.m_pGameInfoObj->m_RoundStartTick;
-				s_Time = (int64)(mix<double>(
-							 0,
-							 (CurTick - MinTick),
-							 pThis->Client()->IntraGameTick(g_Config.m_ClDummy)) *
-						 TickToMicroSeconds) +
+				s_Time = (int64_t)(mix<double>(
+							   0,
+							   (CurTick - MinTick),
+							   pThis->Client()->IntraGameTick(g_Config.m_ClDummy)) *
+						   TickToMicroSeconds) +
 					 MinTick * TickToMicroSeconds;
 			}
 		}
 		else
 		{
-			int64 CurTime = time_get_microseconds();
+			int64_t CurTime = time_get_microseconds();
 			s_Time += CurTime - s_LastLocalTime;
 			s_LastLocalTime = CurTime;
 		}
-		pThis->RenderTools()->RenderEvalEnvelope(pPoints + pItem->m_StartPoint, pItem->m_NumPoints, 4, s_Time + (int64)TimeOffsetMillis * 1000ll, pChannels);
+		CRenderTools::RenderEvalEnvelope(pPoints + pItem->m_StartPoint, pItem->m_NumPoints, 4, s_Time + (int64_t)TimeOffsetMillis * 1000ll, pChannels);
 	}
 }
 
@@ -322,19 +322,11 @@ bool CMapLayers::STileLayerVisuals::Init(unsigned int Width, unsigned int Height
 
 CMapLayers::STileLayerVisuals::~STileLayerVisuals()
 {
-	if(m_TilesOfLayer)
-	{
-		delete[] m_TilesOfLayer;
-	}
-
-	if(m_BorderTop)
-		delete[] m_BorderTop;
-	if(m_BorderBottom)
-		delete[] m_BorderBottom;
-	if(m_BorderLeft)
-		delete[] m_BorderLeft;
-	if(m_BorderRight)
-		delete[] m_BorderRight;
+	delete[] m_TilesOfLayer;
+	delete[] m_BorderTop;
+	delete[] m_BorderBottom;
+	delete[] m_BorderLeft;
+	delete[] m_BorderRight;
 
 	m_TilesOfLayer = NULL;
 	m_BorderTop = NULL;
@@ -402,7 +394,7 @@ void mem_copy_special(void *pDest, void *pSource, size_t Size, size_t Count, siz
 CMapLayers::~CMapLayers()
 {
 	//clear everything and destroy all buffers
-	if(m_TileLayerVisuals.size() != 0)
+	if(!m_TileLayerVisuals.empty())
 	{
 		int s = m_TileLayerVisuals.size();
 		for(int i = 0; i < s; ++i)
@@ -410,7 +402,7 @@ CMapLayers::~CMapLayers()
 			delete m_TileLayerVisuals[i];
 		}
 	}
-	if(m_QuadLayerVisuals.size() != 0)
+	if(!m_QuadLayerVisuals.empty())
 	{
 		int s = m_QuadLayerVisuals.size();
 		for(int i = 0; i < s; ++i)
@@ -425,7 +417,7 @@ void CMapLayers::OnMapLoad()
 	if(!Graphics()->IsTileBufferingEnabled() && !Graphics()->IsQuadBufferingEnabled())
 		return;
 	//clear everything and destroy all buffers
-	if(m_TileLayerVisuals.size() != 0)
+	if(!m_TileLayerVisuals.empty())
 	{
 		int s = m_TileLayerVisuals.size();
 		for(int i = 0; i < s; ++i)
@@ -435,7 +427,7 @@ void CMapLayers::OnMapLoad()
 		}
 		m_TileLayerVisuals.clear();
 	}
-	if(m_QuadLayerVisuals.size() != 0)
+	if(!m_QuadLayerVisuals.empty())
 	{
 		int s = m_QuadLayerVisuals.size();
 		for(int i = 0; i < s; ++i)
@@ -831,8 +823,8 @@ void CMapLayers::OnMapLoad()
 						tmpTileTexCoords.insert(tmpTileTexCoords.end(), tmpBorderRightTilesTexCoords.begin(), tmpBorderRightTilesTexCoords.end());
 
 						//setup params
-						float *pTmpTiles = (tmpTiles.size() == 0) ? NULL : (float *)&tmpTiles[0];
-						unsigned char *pTmpTileTexCoords = (tmpTileTexCoords.size() == 0) ? NULL : (unsigned char *)&tmpTileTexCoords[0];
+						float *pTmpTiles = (tmpTiles.empty()) ? NULL : (float *)&tmpTiles[0];
+						unsigned char *pTmpTileTexCoords = (tmpTileTexCoords.empty()) ? NULL : (unsigned char *)&tmpTileTexCoords[0];
 
 						Visuals.m_BufferContainerIndex = -1;
 						size_t UploadDataSize = tmpTileTexCoords.size() * sizeof(SGraphicTileTexureCoords) + tmpTiles.size() * sizeof(SGraphicTile);
@@ -888,7 +880,7 @@ void CMapLayers::OnMapLoad()
 				m_QuadLayerVisuals.push_back(new SQuadLayerVisuals());
 				SQuadLayerVisuals *pQLayerVisuals = m_QuadLayerVisuals.back();
 
-				bool Textured = (pQLayer->m_Image == -1 ? false : true);
+				bool Textured = (pQLayer->m_Image != -1);
 
 				tmpQuads.clear();
 				tmpQuadsTextured.clear();
@@ -1650,13 +1642,13 @@ void CMapLayers::OnRender()
 					Render = true;
 			}
 
-			if(Render && pLayer->m_Type == LAYERTYPE_TILES && Input()->KeyIsPressed(KEY_LCTRL) && Input()->KeyIsPressed(KEY_LSHIFT) && Input()->KeyPress(KEY_KP_0))
+			if(Render && pLayer->m_Type == LAYERTYPE_TILES && Input()->ModifierIsPressed() && (Input()->KeyIsPressed(KEY_LSHIFT) || Input()->KeyIsPressed(KEY_RSHIFT)) && Input()->KeyPress(KEY_KP_0))
 			{
 				CMapItemLayerTilemap *pTMap = (CMapItemLayerTilemap *)pLayer;
 				CTile *pTiles = (CTile *)m_pLayers->Map()->GetData(pTMap->m_Data);
 				CServerInfo CurrentServerInfo;
 				Client()->GetServerInfo(&CurrentServerInfo);
-				char aFilename[256];
+				char aFilename[IO_MAX_PATH_LENGTH];
 				str_format(aFilename, sizeof(aFilename), "dumps/tilelayer_dump_%s-%d-%d-%dx%d.txt", CurrentServerInfo.m_aMap, g, l, pTMap->m_Width, pTMap->m_Height);
 				IOHANDLE File = Storage()->OpenFile(aFilename, IOFLAG_WRITE, IStorage::TYPE_SAVE);
 				if(File)
