@@ -7,6 +7,8 @@
 
 #include <game/client/component.h>
 
+struct CSoundSource;
+
 class CMapSounds : public CComponent
 {
 	int m_aSounds[64];
@@ -28,10 +30,11 @@ class CMapSounds : public CComponent
 
 public:
 	CMapSounds();
+	virtual int Sizeof() const override { return sizeof(*this); }
 
-	virtual void OnMapLoad();
-	virtual void OnRender();
-	virtual void OnStateChange(int NewState, int OldState);
+	virtual void OnMapLoad() override;
+	virtual void OnRender() override;
+	virtual void OnStateChange(int NewState, int OldState) override;
 };
 
 #endif // GAME_CLIENT_COMPONENTS_MAPSOUNDS_H
