@@ -7,18 +7,19 @@
 class CMotd : public CComponent
 {
 	// motd
-	int64 m_ServerMotdTime;
+	int64_t m_ServerMotdTime;
 
 public:
 	char m_aServerMotd[900];
 
+	virtual int Sizeof() const override { return sizeof(*this); }
 	void Clear();
 	bool IsActive();
 
-	virtual void OnRender();
-	virtual void OnStateChange(int NewState, int OldState);
-	virtual void OnMessage(int MsgType, void *pRawMsg);
-	virtual bool OnInput(IInput::CEvent Event);
+	virtual void OnRender() override;
+	virtual void OnStateChange(int NewState, int OldState) override;
+	virtual void OnMessage(int MsgType, void *pRawMsg) override;
+	virtual bool OnInput(IInput::CEvent Event) override;
 };
 
 #endif
