@@ -9,6 +9,8 @@
 #include <game/gamecore.h>
 #include <game/generated/client_data.h>
 
+#include <game/client/gameclient.h>
+
 CParticles::CParticles()
 {
 	OnReset();
@@ -103,7 +105,7 @@ void CParticles::Update(float TimePassed)
 
 			// move the point
 			vec2 Vel = m_aParticles[i].m_Vel * TimePassed;
-			Collision()->MovePoint(&m_aParticles[i].m_Pos, &Vel, 0.1f + 0.9f * frandom(), NULL);
+			Collision()->MovePoint(&m_aParticles[i].m_Pos, &Vel, 0.1f + 0.9f * random_float(), NULL);
 			m_aParticles[i].m_Vel = Vel * (1.0f / TimePassed);
 
 			m_aParticles[i].m_Life += TimePassed;
@@ -140,8 +142,8 @@ void CParticles::OnRender()
 		return;
 
 	set_new_tick();
-	static int64 LastTime = 0;
-	int64 t = time();
+	static int64_t LastTime = 0;
+	int64_t t = time();
 
 	if(Client()->State() == IClient::STATE_DEMOPLAYBACK)
 	{
@@ -163,13 +165,14 @@ void CParticles::OnInit()
 	Graphics()->QuadsSetRotation(0);
 	Graphics()->SetColor(1.f, 1.f, 1.f, 1.f);
 
-	m_ParticleQuadContainerIndex = Graphics()->CreateQuadContainer();
+	m_ParticleQuadContainerIndex = Graphics()->CreateQuadContainer(false);
 
 	for(int i = 0; i <= (SPRITE_PART9 - SPRITE_PART_SLICE); ++i)
 	{
 		Graphics()->QuadsSetSubset(0, 0, 1, 1);
 		RenderTools()->QuadContainerAddSprite(m_ParticleQuadContainerIndex, 1.f);
 	}
+	Graphics()->QuadContainerUpload(m_ParticleQuadContainerIndex);
 }
 
 bool CParticles::ParticleIsVisibleOnScreen(const vec2 &CurPos, float CurSize)
