@@ -4,6 +4,9 @@
 #define GAME_CLIENT_COMPONENTS_PLAYERS_H
 #include <game/client/component.h>
 
+#include <game/client/render.h>
+#include <game/generated/protocol.h>
+
 class CPlayers : public CComponent
 {
 	friend class CGhost;
@@ -30,8 +33,9 @@ class CPlayers : public CComponent
 
 public:
 	vec2 m_CurPredictedPos[MAX_CLIENTS];
-	virtual void OnInit();
-	virtual void OnRender();
+	virtual int Sizeof() const override { return sizeof(*this); }
+	virtual void OnInit() override;
+	virtual void OnRender() override;
 };
 
 #endif
