@@ -28,7 +28,7 @@ class CSounds : public CComponent
 		int m_SetId;
 	} m_aQueue[QUEUE_SIZE];
 	int m_QueuePos;
-	int64 m_QueueWaitTime;
+	int64_t m_QueueWaitTime;
 	std::shared_ptr<CSoundLoading> m_pSoundJob;
 	bool m_WaitForSoundJob;
 
@@ -50,10 +50,11 @@ public:
 		CHN_MAPSOUND,
 	};
 
-	virtual void OnInit();
-	virtual void OnReset();
-	virtual void OnStateChange(int NewState, int OldState);
-	virtual void OnRender();
+	virtual int Sizeof() const override { return sizeof(*this); }
+	virtual void OnInit() override;
+	virtual void OnReset() override;
+	virtual void OnStateChange(int NewState, int OldState) override;
+	virtual void OnRender() override;
 
 	void ClearQueue();
 	void Enqueue(int Channel, int SetId);
