@@ -8,7 +8,7 @@ class CStatboard : public CComponent
 private:
 	bool m_Active;
 	bool m_ScreenshotTaken;
-	int64 m_ScreenshotTime;
+	int64_t m_ScreenshotTime;
 	static void ConKeyStats(IConsole::IResult *pResult, void *pUserData);
 	void RenderGlobalStats();
 	void AutoStatScreenshot();
@@ -20,11 +20,12 @@ private:
 
 public:
 	CStatboard();
-	virtual void OnReset();
-	virtual void OnConsoleInit();
-	virtual void OnRender();
-	virtual void OnRelease();
-	virtual void OnMessage(int MsgType, void *pRawMsg);
+	virtual int Sizeof() const override { return sizeof(*this); }
+	virtual void OnReset() override;
+	virtual void OnConsoleInit() override;
+	virtual void OnRender() override;
+	virtual void OnRelease() override;
+	virtual void OnMessage(int MsgType, void *pRawMsg) override;
 	bool IsActive();
 };
 
