@@ -20,6 +20,41 @@
 #include <game/client/prediction/entities/pickup.h>
 #include <game/client/prediction/gameworld.h>
 
+// components
+#include "components/background.h"
+#include "components/binds.h"
+#include "components/broadcast.h"
+#include "components/camera.h"
+#include "components/chat.h"
+#include "components/console.h"
+#include "components/controls.h"
+#include "components/countryflags.h"
+#include "components/damageind.h"
+#include "components/debughud.h"
+#include "components/effects.h"
+#include "components/emoticon.h"
+#include "components/flow.h"
+#include "components/ghost.h"
+#include "components/hud.h"
+#include "components/items.h"
+#include "components/killmessages.h"
+#include "components/mapimages.h"
+#include "components/maplayers.h"
+#include "components/mapsounds.h"
+#include "components/menu_background.h"
+#include "components/menus.h"
+#include "components/motd.h"
+#include "components/nameplates.h"
+#include "components/particles.h"
+#include "components/players.h"
+#include "components/race_demo.h"
+#include "components/scoreboard.h"
+#include "components/skins.h"
+#include "components/sounds.h"
+#include "components/spectator.h"
+#include "components/statboard.h"
+#include "components/voting.h"
+
 class CGameInfo
 {
 public:
@@ -57,8 +92,58 @@ public:
 	bool m_AllowXSkins;
 };
 
+class CSnapEntities
+{
+public:
+	IClient::CSnapItem m_Item;
+	const void *m_pData;
+	const CNetObj_EntityEx *m_pDataEx;
+};
+
 class CGameClient : public IGameClient
 {
+public:
+	// all components
+	CKillMessages m_KillMessages;
+	CCamera m_Camera;
+	CChat m_Chat;
+	CMotd m_Motd;
+	CBroadcast m_Broadcast;
+	CGameConsole m_GameConsole;
+	CBinds m_Binds;
+	CParticles m_Particles;
+	CMenus m_Menus;
+	CSkins m_Skins;
+	CCountryFlags m_CountryFlags;
+	CFlow m_Flow;
+	CHud m_Hud;
+	CDebugHud m_DebugHud;
+	CControls m_Controls;
+	CEffects m_Effects;
+	CScoreboard m_Scoreboard;
+	CStatboard m_Statboard;
+	CSounds m_Sounds;
+	CEmoticon m_Emoticon;
+	CDamageInd m_DamageInd;
+	CVoting m_Voting;
+	CSpectator m_Spectator;
+
+	CPlayers m_Players;
+	CNamePlates m_NamePlates;
+	CItems m_Items;
+	CMapImages m_MapImages;
+
+	CMapLayers m_MapLayersBackGround = CMapLayers{CMapLayers::TYPE_BACKGROUND};
+	CMapLayers m_MapLayersForeGround = CMapLayers{CMapLayers::TYPE_FOREGROUND};
+	CBackground m_BackGround;
+	CMenuBackground m_MenuBackground;
+
+	CMapSounds m_MapSounds;
+
+	CRaceDemo m_RaceDemo;
+	CGhost m_Ghost;
+
+private:
 	class CStack
 	{
 	public:
@@ -257,7 +342,8 @@ public:
 		int m_SkinColor;
 		int m_Team;
 		int m_Emoticon;
-		int m_EmoticonStart;
+		float m_EmoticonStartFraction;
+		int m_EmoticonStartTick;
 		bool m_Solo;
 		bool m_Jetpack;
 		bool m_NoCollision;
@@ -274,6 +360,7 @@ public:
 		bool m_HasTelegunLaser;
 		int m_FreezeEnd;
 		bool m_DeepFrozen;
+		bool m_LiveFrozen;
 
 		CCharacterCore m_Predicted;
 		CCharacterCore m_PrevPredicted;
@@ -293,10 +380,13 @@ public:
 		bool m_Paused;
 		bool m_Spec;
 
+		// Editor allows 256 switches for now.
+		bool m_SwitchStates[256];
+
 		CNetObj_Character m_Snapped;
 		CNetObj_Character m_Evolved;
 
-		void UpdateRenderInfo();
+		void UpdateRenderInfo(bool IsTeamPlay);
 		void Reset();
 
 		// rendered characters
@@ -305,8 +395,8 @@ public:
 		vec2 m_RenderPos;
 		bool m_IsPredicted;
 		bool m_IsPredictedLocal;
-		int64 m_SmoothStart[2];
-		int64 m_SmoothLen[2];
+		int64_t m_SmoothStart[2];
+		int64_t m_SmoothLen[2];
 		vec2 m_PredPos[200];
 		int m_PredTick[200];
 		bool m_SpecCharPresent;
@@ -367,7 +457,7 @@ public:
 	virtual void OnInit();
 	virtual void OnConsoleInit();
 	virtual void OnStateChange(int NewState, int OldState);
-	virtual void OnMessage(int MsgId, CUnpacker *pUnpacker, bool IsDummy = 0);
+	virtual void OnMessage(int MsgId, CUnpacker *pUnpacker, int Conn, bool Dummy);
 	virtual void InvalidateSnapshot();
 	virtual void OnNewSnapshot();
 	virtual void OnPredict();
@@ -400,34 +490,6 @@ public:
 	virtual void SendDummyInfo(bool Start);
 	void SendKill(int ClientID);
 
-	// pointers to all systems
-	class CMenuBackground *m_pMenuBackground;
-	class CGameConsole *m_pGameConsole;
-	class CBinds *m_pBinds;
-	class CParticles *m_pParticles;
-	class CMenus *m_pMenus;
-	class CSkins *m_pSkins;
-	class CCountryFlags *m_pCountryFlags;
-	class CFlow *m_pFlow;
-	class CChat *m_pChat;
-	class CDamageInd *m_pDamageind;
-	class CCamera *m_pCamera;
-	class CControls *m_pControls;
-	class CEffects *m_pEffects;
-	class CSounds *m_pSounds;
-	class CMotd *m_pMotd;
-	class CMapImages *m_pMapimages;
-	class CVoting *m_pVoting;
-	class CScoreboard *m_pScoreboard;
-	class CStatboard *m_pStatboard;
-	class CItems *m_pItems;
-	class CMapLayers *m_pMapLayersBackGround;
-	class CMapLayers *m_pMapLayersForeGround;
-	class CBackground *m_pBackGround;
-
-	class CMapSounds *m_pMapSounds;
-	class CPlayers *m_pPlayers;
-
 	// DDRace
 
 	int m_LocalIDs[NUM_DUMMIES];
@@ -436,14 +498,14 @@ public:
 	int m_DummyFire;
 	bool m_ReceivedDDNetPlayer;
 
-	class CRaceDemo *m_pRaceDemo;
-	class CGhost *m_pGhost;
 	class CTeamsCore m_Teams;
 
-	int IntersectCharacter(vec2 Pos0, vec2 Pos1, vec2 &NewPos, int ownID);
+	int IntersectCharacter(vec2 HookPos, vec2 NewPos, vec2 &NewPos2, int ownID);
 
 	virtual int GetLastRaceTick();
 
+	bool IsTeamPlay() { return m_Snap.m_pGameInfoObj && m_Snap.m_pGameInfoObj->m_GameFlags & GAMEFLAG_TEAMS; }
+
 	bool AntiPingPlayers() { return g_Config.m_ClAntiPing && g_Config.m_ClAntiPingPlayers && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK && (m_Tuning[g_Config.m_ClDummy].m_PlayerCollision || m_Tuning[g_Config.m_ClDummy].m_PlayerHooking); }
 	bool AntiPingGrenade() { return g_Config.m_ClAntiPing && g_Config.m_ClAntiPingGrenade && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK; }
 	bool AntiPingWeapons() { return g_Config.m_ClAntiPing && g_Config.m_ClAntiPingWeapons && !m_Snap.m_SpecInfo.m_Active && Client()->State() != IClient::STATE_DEMOPLAYBACK; }
@@ -459,6 +521,8 @@ public:
 	void DummyResetInput();
 	void Echo(const char *pString);
 	bool IsOtherTeam(int ClientID);
+	int SwitchStateTeam();
+	bool IsLocalCharSuper();
 	bool CanDisplayWarning();
 	bool IsDisplayingWarning();
 
@@ -545,7 +609,7 @@ public:
 
 		bool IsSixup()
 		{
-			return m_SpriteNinjaBarFullLeft != -1;
+			return m_SpriteNinjaBarFullLeft.IsValid();
 		}
 	};
 
@@ -576,7 +640,12 @@ public:
 	SClientEmoticonsSkin m_EmoticonsSkin;
 	bool m_EmoticonsSkinLoaded;
 
+	const std::vector<CSnapEntities> &SnapEntities() { return m_aSnapEntities; }
+
 private:
+	std::vector<CSnapEntities> m_aSnapEntities;
+	void SnapCollectEntities();
+
 	bool m_DDRaceMsgSent[NUM_DUMMIES];
 	int m_ShowOthers[NUM_DUMMIES];
 
@@ -589,6 +658,7 @@ private:
 	int m_IsDummySwapping;
 	CCharOrder m_CharOrder;
 	class CCharacter m_aLastWorldCharacters[MAX_CLIENTS];
+	int m_SwitchStateTeam[NUM_DUMMIES];
 
 	enum
 	{
