@@ -13,7 +13,7 @@ class CLineInput
 		MAX_SIZE = 512,
 		MAX_CHARS = MAX_SIZE / 2,
 	};
-	char m_Str[MAX_SIZE];
+	char m_aStr[MAX_SIZE];
 	int m_Len;
 	int m_CursorPos;
 	int m_NumChars;
@@ -23,27 +23,35 @@ class CLineInput
 	int m_FakeCursorPos;
 
 public:
-	static bool Manipulate(IInput::CEvent e, char *pStr, int StrMaxSize, int StrMaxChars, int *pStrLenPtr, int *pCursorPosPtr, int *pNumCharsPtr);
-
-	class CCallback
+	enum ELineInputChanges
 	{
-	public:
-		virtual ~CCallback() {}
-		virtual bool Event(IInput::CEvent e) = 0;
+		// string was changed
+		LINE_INPUT_CHANGE_STRING = 1 << 0,
+		// characters were removed from the string
+		LINE_INPUT_CHANGE_CHARACTERS_DELETE = 1 << 1,
+		// cursor was changed or tried to change(e.g. pressing right at the last char in the string)
+		LINE_INPUT_CHANGE_CURSOR = 1 << 2,
+		LINE_INPUT_CHANGE_WARP_CURSOR = 1 << 3,
 	};
+	enum ELineInputModifyFlags
+	{
+		// don't delete characters
+		LINE_INPUT_MODIFY_DONT_DELETE = 1 << 0,
+	};
+	static int32_t Manipulate(IInput::CEvent e, char *pStr, int StrMaxSize, int StrMaxChars, int *pStrLenPtr, int *pCursorPosPtr, int *pNumCharsPtr, int32_t ModifyFlags, int ModifierKey);
 
 	CLineInput();
 	void Clear();
 	void ProcessInput(IInput::CEvent e);
 	void Editing(const char *pString, int Cursor);
 	void Set(const char *pString);
-	void Add(const char *pString);
-	const char *GetString(bool Editing = false) const { return Editing ? m_DisplayStr : m_Str; }
+	void SetRange(const char *pString, int Begin, int End);
+	void Insert(const char *pString, int Begin);
+	void Append(const char *pString);
+	const char *GetString(bool Editing = false) const { return Editing ? m_DisplayStr : m_aStr; }
 	int GetLength(bool Editing = false) const { return Editing ? m_FakeLen : m_Len; }
 	int GetCursorOffset(bool Editing = false) const { return Editing ? m_FakeCursorPos : m_CursorPos; }
 	void SetCursorOffset(int Offset) { m_CursorPos = Offset > m_Len ? m_Len : Offset < 0 ? 0 : Offset; }
-	void DeleteUntilCursor();
-	void DeleteFromCursor();
 };
 
 #endif
