@@ -287,15 +287,15 @@ CTuningParams *CGameWorld::Tuning()
 	return &m_Core.m_Tuning[g_Config.m_ClDummy];
 }
 
-CEntity *CGameWorld::GetEntity(int ID, int EntType)
+CEntity *CGameWorld::GetEntity(int ID, int EntityType)
 {
-	for(CEntity *pEnt = m_apFirstEntityTypes[EntType]; pEnt; pEnt = pEnt->m_pNextTypeEntity)
+	for(CEntity *pEnt = m_apFirstEntityTypes[EntityType]; pEnt; pEnt = pEnt->m_pNextTypeEntity)
 		if(pEnt->m_ID == ID)
 			return pEnt;
 	return 0;
 }
 
-void CGameWorld::CreateExplosion(vec2 Pos, int Owner, int Weapon, bool NoDamage, int ActivatedTeam, int64 Mask)
+void CGameWorld::CreateExplosion(vec2 Pos, int Owner, int Weapon, bool NoDamage, int ActivatedTeam, int64_t Mask)
 {
 	if(Owner < 0 && m_WorldConfig.m_IsSolo && !(Weapon == WEAPON_SHOTGUN && m_WorldConfig.m_IsDDRace))
 		return;
@@ -362,7 +362,7 @@ void CGameWorld::NetCharAdd(int ObjID, CNetObj_Character *pCharObj, CNetObj_DDNe
 		pChar->m_GameTeam = GameTeam;
 }
 
-void CGameWorld::NetObjAdd(int ObjID, int ObjType, const void *pObjData)
+void CGameWorld::NetObjAdd(int ObjID, int ObjType, const void *pObjData, const CNetObj_EntityEx *pDataEx)
 {
 	if((ObjType == NETOBJTYPE_PROJECTILE || ObjType == NETOBJTYPE_DDNETPROJECTILE) && m_WorldConfig.m_PredictWeapons)
 	{
@@ -375,7 +375,7 @@ void CGameWorld::NetObjAdd(int ObjID, int ObjType, const void *pObjData)
 		{
 			Data = ExtractProjectileInfoDDNet((const CNetObj_DDNetProjectile *)pObjData, this);
 		}
-		CProjectile NetProj = CProjectile(this, ObjID, &Data);
+		CProjectile NetProj = CProjectile(this, ObjID, &Data, pDataEx);
 
 		if(NetProj.m_Type != WEAPON_SHOTGUN && fabs(length(NetProj.m_Direction) - 1.f) > 0.02f) // workaround to skip grenades on ball mod
 			return;
@@ -429,7 +429,7 @@ void CGameWorld::NetObjAdd(int ObjID, int ObjType, const void *pObjData)
 	}
 	else if(ObjType == NETOBJTYPE_PICKUP && m_WorldConfig.m_PredictWeapons)
 	{
-		CPickup NetPickup = CPickup(this, ObjID, (CNetObj_Pickup *)pObjData);
+		CPickup NetPickup = CPickup(this, ObjID, (CNetObj_Pickup *)pObjData, pDataEx);
 		if(CPickup *pPickup = (CPickup *)GetEntity(ObjID, ENTTYPE_PICKUP))
 		{
 			if(NetPickup.Match(pPickup))
@@ -446,14 +446,15 @@ void CGameWorld::NetObjAdd(int ObjID, int ObjType, const void *pObjData)
 	{
 		CLaser NetLaser = CLaser(this, ObjID, (CNetObj_Laser *)pObjData);
 		CLaser *pMatching = 0;
-		if(CLaser *pLaser = (CLaser *)GetEntity(ObjID, ENTTYPE_LASER))
+		if(CLaser *pLaser = dynamic_cast<CLaser *>(GetEntity(ObjID, ENTTYPE_LASER)))
 			if(NetLaser.Match(pLaser))
 				pMatching = pLaser;
 		if(!pMatching)
 		{
-			for(CLaser *pLaser = (CLaser *)FindFirst(CGameWorld::ENTTYPE_LASER); pLaser; pLaser = (CLaser *)pLaser->TypeNext())
+			for(CEntity *pEnt = FindFirst(CGameWorld::ENTTYPE_LASER); pEnt; pEnt = pEnt->TypeNext())
 			{
-				if(pLaser->m_ID == -1 && NetLaser.Match(pLaser))
+				auto *const pLaser = dynamic_cast<CLaser *>(pEnt);
+				if(pLaser && pLaser->m_ID == -1 && NetLaser.Match(pLaser))
 				{
 					pMatching = pLaser;
 					pMatching->m_ID = ObjID;
