@@ -19,32 +19,12 @@
 static float gs_SpriteWScale;
 static float gs_SpriteHScale;
 
-/*
-static void layershot_begin()
-{
-	if(!config.cl_layershot)
-		return;
-
-	Graphics()->Clear(0,0,0);
-}
-
-static void layershot_end()
-{
-	if(!config.cl_layershot)
-		return;
-
-	char buf[256];
-	str_format(buf, sizeof(buf), "screenshots/layers_%04d.png", config.cl_layershot);
-	gfx_screenshot_direct(buf);
-	config.cl_layershot++;
-}*/
-
 void CRenderTools::Init(IGraphics *pGraphics, CUI *pUI, CGameClient *pGameClient)
 {
 	m_pGraphics = pGraphics;
 	m_pUI = pUI;
 	m_pGameClient = (CGameClient *)pGameClient;
-	m_TeeQuadContainerIndex = Graphics()->CreateQuadContainer();
+	m_TeeQuadContainerIndex = Graphics()->CreateQuadContainer(false);
 	Graphics()->SetColor(1.f, 1.f, 1.f, 1.f);
 
 	Graphics()->QuadsSetSubset(0, 0, 1, 1);
@@ -67,6 +47,7 @@ void CRenderTools::Init(IGraphics *pGraphics, CUI *pUI, CGameClient *pGameClient
 	QuadContainerAddSprite(m_TeeQuadContainerIndex, -32.f, -16.f, 64.f, 32.f);
 	Graphics()->QuadsSetSubset(0, 0, 1, 1);
 	QuadContainerAddSprite(m_TeeQuadContainerIndex, -32.f, -16.f, 64.f, 32.f);
+	Graphics()->QuadContainerUpload(m_TeeQuadContainerIndex);
 }
 
 void CRenderTools::SelectSprite(CDataSprite *pSpr, int Flags, int sx, int sy)
@@ -389,7 +370,7 @@ void CRenderTools::DrawRoundRectExt4(float x, float y, float w, float h, vec4 Co
 
 int CRenderTools::CreateRoundRectQuadContainer(float x, float y, float w, float h, float r, int Corners)
 {
-	int ContainerIndex = Graphics()->CreateQuadContainer();
+	int ContainerIndex = Graphics()->CreateQuadContainer(false);
 
 	IGraphics::CFreeformItem ArrayF[32];
 	int NumItems = 0;
@@ -434,7 +415,9 @@ int CRenderTools::CreateRoundRectQuadContainer(float x, float y, float w, float
 				x + w - r + Ca3 * r, y + h - r + Sa3 * r,
 				x + w - r + Ca2 * r, y + h - r + Sa2 * r);
 	}
-	Graphics()->QuadContainerAddQuads(ContainerIndex, ArrayF, NumItems);
+
+	if(NumItems > 0)
+		Graphics()->QuadContainerAddQuads(ContainerIndex, ArrayF, NumItems);
 
 	IGraphics::CQuadItem ArrayQ[9];
 	NumItems = 0;
@@ -453,7 +436,11 @@ int CRenderTools::CreateRoundRectQuadContainer(float x, float y, float w, float
 	if(!(Corners & 8))
 		ArrayQ[NumItems++] = IGraphics::CQuadItem(x + w, y + h, -r, -r); // BR
 
-	Graphics()->QuadContainerAddQuads(ContainerIndex, ArrayQ, NumItems);
+	if(NumItems > 0)
+		Graphics()->QuadContainerAddQuads(ContainerIndex, ArrayQ, NumItems);
+
+	Graphics()->QuadContainerUpload(ContainerIndex);
+	Graphics()->QuadContainerChangeAutomaticUpload(ContainerIndex, true);
 
 	return ContainerIndex;
 }
@@ -461,7 +448,7 @@ int CRenderTools::CreateRoundRectQuadContainer(float x, float y, float w, float
 void CRenderTools::DrawUIElRect(CUIElement::SUIElementRect &ElUIRect, const CUIRect *pRect, ColorRGBA Color, int Corners, float Rounding)
 {
 	bool NeedsRecreate = false;
-	if(ElUIRect.m_UIRectQuadContainer == -1 || ElUIRect.m_X != pRect->x || ElUIRect.m_Y != pRect->y || ElUIRect.m_Width != pRect->w || ElUIRect.m_Height != pRect->h)
+	if(ElUIRect.m_UIRectQuadContainer == -1 || ElUIRect.m_X != pRect->x || ElUIRect.m_Y != pRect->y || ElUIRect.m_Width != pRect->w || ElUIRect.m_Height != pRect->h || mem_comp(&ElUIRect.m_QuadColor, &Color, sizeof(Color)) != 0)
 	{
 		if(ElUIRect.m_UIRectQuadContainer != -1)
 			Graphics()->DeleteQuadContainer(ElUIRect.m_UIRectQuadContainer);
@@ -473,6 +460,7 @@ void CRenderTools::DrawUIElRect(CUIElement::SUIElementRect &ElUIRect, const CUIR
 		ElUIRect.m_Y = pRect->y;
 		ElUIRect.m_Width = pRect->w;
 		ElUIRect.m_Height = pRect->h;
+		ElUIRect.m_QuadColor = Color;
 
 		Graphics()->SetColor(Color);
 		ElUIRect.m_UIRectQuadContainer = CreateRoundRectQuadContainer(pRect->x, pRect->y, pRect->w, pRect->h, Rounding, Corners);
@@ -488,34 +476,34 @@ void CRenderTools::DrawRoundRect(float x, float y, float w, float h, float r)
 	DrawRoundRectExt(x, y, w, h, r, 0xf);
 }
 
-void CRenderTools::DrawUIRect(const CUIRect *r, ColorRGBA Color, int Corners, float Rounding)
+void CRenderTools::DrawUIRect(const CUIRect *pRect, ColorRGBA Color, int Corners, float Rounding)
 {
 	Graphics()->TextureClear();
 
 	// TODO: FIX US
 	Graphics()->QuadsBegin();
 	Graphics()->SetColor(Color);
-	DrawRoundRectExt(r->x, r->y, r->w, r->h, Rounding * UI()->Scale(), Corners);
+	DrawRoundRectExt(pRect->x, pRect->y, pRect->w, pRect->h, Rounding * UI()->Scale(), Corners);
 	Graphics()->QuadsEnd();
 }
 
-void CRenderTools::DrawUIRect4(const CUIRect *r, vec4 ColorTopLeft, vec4 ColorTopRight, vec4 ColorBottomLeft, vec4 ColorBottomRight, int Corners, float Rounding)
+void CRenderTools::DrawUIRect4(const CUIRect *pRect, vec4 ColorTopLeft, vec4 ColorTopRight, vec4 ColorBottomLeft, vec4 ColorBottomRight, int Corners, float Rounding)
 {
 	Graphics()->TextureClear();
 
 	Graphics()->QuadsBegin();
-	DrawRoundRectExt4(r->x, r->y, r->w, r->h, ColorTopLeft, ColorTopRight, ColorBottomLeft, ColorBottomRight, Rounding, Corners);
+	DrawRoundRectExt4(pRect->x, pRect->y, pRect->w, pRect->h, ColorTopLeft, ColorTopRight, ColorBottomLeft, ColorBottomRight, Rounding, Corners);
 	Graphics()->QuadsEnd();
 }
 
-void CRenderTools::DrawUIRect4NoRounding(const CUIRect *r, vec4 ColorTopLeft, vec4 ColorTopRight, vec4 ColorBottomLeft, vec4 ColorBottomRight)
+void CRenderTools::DrawUIRect4NoRounding(const CUIRect *pRect, vec4 ColorTopLeft, vec4 ColorTopRight, vec4 ColorBottomLeft, vec4 ColorBottomRight)
 {
 	Graphics()->TextureClear();
 
 	Graphics()->QuadsBegin();
 
 	Graphics()->SetColor4(ColorTopLeft, ColorTopRight, ColorBottomLeft, ColorBottomRight);
-	IGraphics::CQuadItem ItemQ = IGraphics::CQuadItem(r->x, r->y, r->w, r->h);
+	IGraphics::CQuadItem ItemQ = IGraphics::CQuadItem(pRect->x, pRect->y, pRect->w, pRect->h);
 	Graphics()->QuadsDrawTL(&ItemQ, 1);
 
 	Graphics()->QuadsEnd();
@@ -786,37 +774,3 @@ void CRenderTools::MapscreenToWorld(float CenterX, float CenterY, float Parallax
 	pPoints[2] = pPoints[0] + Width;
 	pPoints[3] = pPoints[1] + Height;
 }
-
-void CRenderTools::RenderTilemapGenerateSkip(class CLayers *pLayers)
-{
-	for(int g = 0; g < pLayers->NumGroups(); g++)
-	{
-		CMapItemGroup *pGroup = pLayers->GetGroup(g);
-
-		for(int l = 0; l < pGroup->m_NumLayers; l++)
-		{
-			CMapItemLayer *pLayer = pLayers->GetLayer(pGroup->m_StartLayer + l);
-
-			if(pLayer->m_Type == LAYERTYPE_TILES)
-			{
-				CMapItemLayerTilemap *pTmap = (CMapItemLayerTilemap *)pLayer;
-				CTile *pTiles = (CTile *)pLayers->Map()->GetData(pTmap->m_Data);
-				for(int y = 0; y < pTmap->m_Height; y++)
-				{
-					for(int x = 1; x < pTmap->m_Width;)
-					{
-						int sx;
-						for(sx = 1; x + sx < pTmap->m_Width && sx < 255; sx++)
-						{
-							if(pTiles[y * pTmap->m_Width + x + sx].m_Index)
-								break;
-						}
-
-						pTiles[y * pTmap->m_Width + x].m_Skip = sx - 1;
-						x += sx;
-					}
-				}
-			}
-		}
-	}
-}
