@@ -11,7 +11,7 @@
 #include <game/generated/client_data.h>
 #include <game/generated/protocol.h>
 
-void CRenderTools::RenderEvalEnvelope(CEnvPoint *pPoints, int NumPoints, int Channels, int64 TimeMicros, float *pResult)
+void CRenderTools::RenderEvalEnvelope(CEnvPoint *pPoints, int NumPoints, int Channels, int64_t TimeMicros, float *pResult)
 {
 	if(NumPoints == 0)
 	{
@@ -31,7 +31,7 @@ void CRenderTools::RenderEvalEnvelope(CEnvPoint *pPoints, int NumPoints, int Cha
 		return;
 	}
 
-	int64 MaxPointTime = (int64)pPoints[NumPoints - 1].m_Time * 1000ll;
+	int64_t MaxPointTime = (int64_t)pPoints[NumPoints - 1].m_Time * 1000ll;
 	if(MaxPointTime > 0) // TODO: remove this check when implementing a IO check for maps(in this case broken envelopes)
 		TimeMicros = TimeMicros % MaxPointTime;
 	else
@@ -76,7 +76,6 @@ void CRenderTools::RenderEvalEnvelope(CEnvPoint *pPoints, int NumPoints, int Cha
 	pResult[1] = fx2f(pPoints[NumPoints - 1].m_aValues[1]);
 	pResult[2] = fx2f(pPoints[NumPoints - 1].m_aValues[2]);
 	pResult[3] = fx2f(pPoints[NumPoints - 1].m_aValues[3]);
-	return;
 }
 
 static void Rotate(CPoint *pCenter, CPoint *pPoint, float Rotation)
