@@ -16,24 +16,27 @@ class CLayers
 	CMapItemLayerTilemap *m_pGameLayer;
 	class IMap *m_pMap;
 
+	void InitTilemapSkip();
+
 public:
 	CLayers();
 	void Init(class IKernel *pKernel);
 	void InitBackground(class IMap *pMap);
-	int NumGroups() const { return m_GroupsNum; };
-	class IMap *Map() const { return m_pMap; };
-	CMapItemGroup *GameGroup() const { return m_pGameGroup; };
-	CMapItemLayerTilemap *GameLayer() const { return m_pGameLayer; };
+	int NumGroups() const { return m_GroupsNum; }
+	int NumLayers() const { return m_LayersNum; }
+	class IMap *Map() const { return m_pMap; }
+	CMapItemGroup *GameGroup() const { return m_pGameGroup; }
+	CMapItemLayerTilemap *GameLayer() const { return m_pGameLayer; }
 	CMapItemGroup *GetGroup(int Index) const;
 	CMapItemLayer *GetLayer(int Index) const;
 
 	// DDRace
 
-	CMapItemLayerTilemap *TeleLayer() const { return m_pTeleLayer; };
-	CMapItemLayerTilemap *SpeedupLayer() const { return m_pSpeedupLayer; };
-	CMapItemLayerTilemap *FrontLayer() const { return m_pFrontLayer; };
-	CMapItemLayerTilemap *SwitchLayer() const { return m_pSwitchLayer; };
-	CMapItemLayerTilemap *TuneLayer() const { return m_pTuneLayer; };
+	CMapItemLayerTilemap *TeleLayer() const { return m_pTeleLayer; }
+	CMapItemLayerTilemap *SpeedupLayer() const { return m_pSpeedupLayer; }
+	CMapItemLayerTilemap *FrontLayer() const { return m_pFrontLayer; }
+	CMapItemLayerTilemap *SwitchLayer() const { return m_pSwitchLayer; }
+	CMapItemLayerTilemap *TuneLayer() const { return m_pTuneLayer; }
 
 private:
 	CMapItemLayerTilemap *m_pTeleLayer;
