@@ -169,6 +169,8 @@ enum
 	TILE_CREDITS_2 = 141,
 	TILE_CREDITS_3 = 142,
 	TILE_CREDITS_4 = 143,
+	TILE_LFREEZE = 144,
+	TILE_LUNFREEZE = 145,
 	TILE_CREDITS_5 = 156,
 	TILE_CREDITS_6 = 157,
 	TILE_CREDITS_7 = 158,
