@@ -277,23 +277,27 @@ void CGameContext::ModifyWeapons(IConsole::IResult *pResult, void *pUserData,
 	pChr->m_DDRaceState = DDRACE_CHEAT;
 }
 
+void CGameContext::Teleport(CCharacter *pChr, vec2 Pos)
+{
+	pChr->Core()->m_Pos = Pos;
+	pChr->m_Pos = Pos;
+	pChr->m_PrevPos = Pos;
+	pChr->m_DDRaceState = DDRACE_CHEAT;
+}
+
 void CGameContext::ConToTeleporter(IConsole::IResult *pResult, void *pUserData)
 {
 	CGameContext *pSelf = (CGameContext *)pUserData;
 	unsigned int TeleTo = pResult->GetInteger(0);
 	CGameControllerDDRace *pGameControllerDDRace = (CGameControllerDDRace *)pSelf->m_pController;
 
-	if(pGameControllerDDRace->m_TeleOuts[TeleTo - 1].size())
+	if(!pGameControllerDDRace->m_TeleOuts[TeleTo - 1].empty())
 	{
 		CCharacter *pChr = pSelf->GetPlayerChar(pResult->m_ClientID);
 		if(pChr)
 		{
 			int TeleOut = pSelf->m_World.m_Core.RandomOr0(pGameControllerDDRace->m_TeleOuts[TeleTo - 1].size());
-			vec2 TelePos = pGameControllerDDRace->m_TeleOuts[TeleTo - 1][TeleOut];
-			pChr->Core()->m_Pos = TelePos;
-			pChr->m_Pos = TelePos;
-			pChr->m_PrevPos = TelePos;
-			pChr->m_DDRaceState = DDRACE_CHEAT;
+			pSelf->Teleport(pChr, pGameControllerDDRace->m_TeleOuts[TeleTo - 1][TeleOut]);
 		}
 	}
 }
@@ -304,17 +308,13 @@ void CGameContext::ConToCheckTeleporter(IConsole::IResult *pResult, void *pUserD
 	unsigned int TeleTo = pResult->GetInteger(0);
 	CGameControllerDDRace *pGameControllerDDRace = (CGameControllerDDRace *)pSelf->m_pController;
 
-	if(pGameControllerDDRace->m_TeleCheckOuts[TeleTo - 1].size())
+	if(!pGameControllerDDRace->m_TeleCheckOuts[TeleTo - 1].empty())
 	{
 		CCharacter *pChr = pSelf->GetPlayerChar(pResult->m_ClientID);
 		if(pChr)
 		{
 			int TeleOut = pSelf->m_World.m_Core.RandomOr0(pGameControllerDDRace->m_TeleCheckOuts[TeleTo - 1].size());
-			vec2 TelePos = pGameControllerDDRace->m_TeleCheckOuts[TeleTo - 1][TeleOut];
-			pChr->Core()->m_Pos = TelePos;
-			pChr->m_Pos = TelePos;
-			pChr->m_PrevPos = TelePos;
-			pChr->m_DDRaceState = DDRACE_CHEAT;
+			pSelf->Teleport(pChr, pGameControllerDDRace->m_TeleCheckOuts[TeleTo - 1][TeleOut]);
 			pChr->m_TeleCheckpoint = TeleTo;
 		}
 	}
@@ -336,10 +336,7 @@ void CGameContext::ConTeleport(IConsole::IResult *pResult, void *pUserData)
 	CCharacter *pChr = pSelf->GetPlayerChar(Tele);
 	if(pChr && pSelf->GetPlayerChar(TeleTo))
 	{
-		pChr->Core()->m_Pos = pSelf->m_apPlayers[TeleTo]->m_ViewPos;
-		pChr->m_Pos = pSelf->m_apPlayers[TeleTo]->m_ViewPos;
-		pChr->m_PrevPos = pSelf->m_apPlayers[TeleTo]->m_ViewPos;
-		pChr->m_DDRaceState = DDRACE_CHEAT;
+		pSelf->Teleport(pChr, pSelf->m_apPlayers[TeleTo]->m_ViewPos);
 	}
 }
 
@@ -434,15 +431,20 @@ bool CGameContext::VoteUnmute(const NETADDR *pAddr, const char *pDisplayName, in
 	return false;
 }
 
-bool CGameContext::TryMute(const NETADDR *pAddr, int Secs, const char *pReason)
+bool CGameContext::TryMute(const NETADDR *pAddr, int Secs, const char *pReason, bool InitialChatDelay)
 {
 	// find a matching mute for this ip, update expiration time if found
 	for(int i = 0; i < m_NumMutes; i++)
 	{
 		if(net_addr_comp_noport(&m_aMutes[i].m_Addr, pAddr) == 0)
 		{
-			m_aMutes[i].m_Expire = Server()->Tick() + Secs * Server()->TickSpeed();
-			str_copy(m_aMutes[i].m_aReason, pReason, sizeof(m_aMutes[i].m_aReason));
+			const int NewExpire = Server()->Tick() + Secs * Server()->TickSpeed();
+			if(NewExpire > m_aMutes[i].m_Expire)
+			{
+				m_aMutes[i].m_Expire = NewExpire;
+				str_copy(m_aMutes[i].m_aReason, pReason, sizeof(m_aMutes[i].m_aReason));
+				m_aMutes[i].m_InitialChatDelay = InitialChatDelay;
+			}
 			return true;
 		}
 	}
@@ -453,6 +455,7 @@ bool CGameContext::TryMute(const NETADDR *pAddr, int Secs, const char *pReason)
 		m_aMutes[m_NumMutes].m_Addr = *pAddr;
 		m_aMutes[m_NumMutes].m_Expire = Server()->Tick() + Secs * Server()->TickSpeed();
 		str_copy(m_aMutes[m_NumMutes].m_aReason, pReason, sizeof(m_aMutes[m_NumMutes].m_aReason));
+		m_aMutes[m_NumMutes].m_InitialChatDelay = InitialChatDelay;
 		m_NumMutes++;
 		return true;
 	}
@@ -461,11 +464,12 @@ bool CGameContext::TryMute(const NETADDR *pAddr, int Secs, const char *pReason)
 	return false;
 }
 
-void CGameContext::Mute(const NETADDR *pAddr, int Secs, const char *pDisplayName, const char *pReason)
+void CGameContext::Mute(const NETADDR *pAddr, int Secs, const char *pDisplayName, const char *pReason, bool InitialChatDelay)
 {
-	if(!TryMute(pAddr, Secs, pReason))
+	if(!TryMute(pAddr, Secs, pReason, InitialChatDelay))
+		return;
+	if(InitialChatDelay)
 		return;
-
 	if(!pDisplayName)
 		return;
 
@@ -601,19 +605,47 @@ void CGameContext::ConMuteIP(IConsole::IResult *pResult, void *pUserData)
 void CGameContext::ConUnmute(IConsole::IResult *pResult, void *pUserData)
 {
 	CGameContext *pSelf = (CGameContext *)pUserData;
-	char aIpBuf[64];
-	char aBuf[64];
-	int Victim = pResult->GetVictim();
+	int Index = pResult->GetInteger(0);
 
-	if(Victim < 0 || Victim >= pSelf->m_NumMutes)
+	if(Index < 0 || Index >= pSelf->m_NumMutes)
 		return;
 
-	pSelf->m_NumMutes--;
-	pSelf->m_aMutes[Victim] = pSelf->m_aMutes[pSelf->m_NumMutes];
-
-	net_addr_str(&pSelf->m_aMutes[Victim].m_Addr, aIpBuf, sizeof(aIpBuf), false);
+	char aIpBuf[64];
+	char aBuf[64];
+	net_addr_str(&pSelf->m_aMutes[Index].m_Addr, aIpBuf, sizeof(aIpBuf), false);
 	str_format(aBuf, sizeof(aBuf), "Unmuted %s", aIpBuf);
 	pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "mutes", aBuf);
+
+	pSelf->m_NumMutes--;
+	pSelf->m_aMutes[Index] = pSelf->m_aMutes[pSelf->m_NumMutes];
+}
+
+// unmute by player id
+void CGameContext::ConUnmuteID(IConsole::IResult *pResult, void *pUserData)
+{
+	CGameContext *pSelf = (CGameContext *)pUserData;
+	int Victim = pResult->GetVictim();
+
+	if(Victim < 0 || Victim > MAX_CLIENTS || !pSelf->m_apPlayers[Victim])
+		return;
+
+	NETADDR Addr;
+	pSelf->Server()->GetClientAddr(Victim, &Addr);
+
+	for(int i = 0; i < pSelf->m_NumMutes; i++)
+	{
+		if(net_addr_comp_noport(&pSelf->m_aMutes[i].m_Addr, &Addr) == 0)
+		{
+			char aIpBuf[64];
+			char aBuf[64];
+			net_addr_str(&pSelf->m_aMutes[i].m_Addr, aIpBuf, sizeof(aIpBuf), false);
+			str_format(aBuf, sizeof(aBuf), "Unmuted %s", aIpBuf);
+			pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "mutes", aBuf);
+			pSelf->m_NumMutes--;
+			pSelf->m_aMutes[i] = pSelf->m_aMutes[pSelf->m_NumMutes];
+			return;
+		}
+	}
 }
 
 // list mutes
