@@ -12,11 +12,11 @@ class CDragger : public CEntity
 	int m_EvalTick;
 	void Move();
 	void Drag();
-	CCharacter *m_Target;
+	int m_TargetID;
 	bool m_NW;
 	int m_CaughtTeam;
 
-	CCharacter *m_SoloEnts[MAX_CLIENTS];
+	int m_SoloEntIDs[MAX_CLIENTS];
 	int m_SoloIDs[MAX_CLIENTS];
 
 public:
@@ -25,7 +25,7 @@ public:
 
 	virtual void Reset();
 	virtual void Tick();
-	virtual void Snap(int snapping_client);
+	virtual void Snap(int SnappingClient);
 };
 
 class CDraggerTeam
