@@ -119,37 +119,47 @@ float IGameController::EvaluateSpawnPos(CSpawnEval *pEval, vec2 Pos)
 
 void IGameController::EvaluateSpawnType(CSpawnEval *pEval, int Type)
 {
-	// get spawn point
-	for(int i = 0; i < m_aNumSpawnPoints[Type]; i++)
+	// j == 0: Find an empty slot, j == 1: Take any slot if no empty one found
+	for(int j = 0; j < 2 && !pEval->m_Got; j++)
 	{
-		// check if the position is occupado
-		CCharacter *aEnts[MAX_CLIENTS];
-		int Num = GameServer()->m_World.FindEntities(m_aaSpawnPoints[Type][i], 64, (CEntity **)aEnts, MAX_CLIENTS, CGameWorld::ENTTYPE_CHARACTER);
-		vec2 Positions[5] = {vec2(0.0f, 0.0f), vec2(-32.0f, 0.0f), vec2(0.0f, -32.0f), vec2(32.0f, 0.0f), vec2(0.0f, 32.0f)}; // start, left, up, right, down
-		int Result = -1;
-		for(int Index = 0; Index < 5 && Result == -1; ++Index)
+		// get spawn point
+		for(int i = 0; i < m_aNumSpawnPoints[Type]; i++)
 		{
-			Result = Index;
-			if(!GameServer()->m_World.m_Core.m_Tuning[0].m_PlayerCollision)
-				break;
-			for(int c = 0; c < Num; ++c)
-				if(GameServer()->Collision()->CheckPoint(m_aaSpawnPoints[Type][i] + Positions[Index]) ||
-					distance(aEnts[c]->m_Pos, m_aaSpawnPoints[Type][i] + Positions[Index]) <= aEnts[c]->GetProximityRadius())
-				{
-					Result = -1;
-					break;
-				}
-		}
-		if(Result == -1)
-			continue; // try next spawn point
+			vec2 P = m_aaSpawnPoints[Type][i];
 
-		vec2 P = m_aaSpawnPoints[Type][i] + Positions[Result];
-		float S = EvaluateSpawnPos(pEval, P);
-		if(!pEval->m_Got || pEval->m_Score > S)
-		{
-			pEval->m_Got = true;
-			pEval->m_Score = S;
-			pEval->m_Pos = P;
+			if(j == 0)
+			{
+				// check if the position is occupado
+				CCharacter *aEnts[MAX_CLIENTS];
+				int Num = GameServer()->m_World.FindEntities(m_aaSpawnPoints[Type][i], 64, (CEntity **)aEnts, MAX_CLIENTS, CGameWorld::ENTTYPE_CHARACTER);
+				vec2 Positions[5] = {vec2(0.0f, 0.0f), vec2(-32.0f, 0.0f), vec2(0.0f, -32.0f), vec2(32.0f, 0.0f), vec2(0.0f, 32.0f)}; // start, left, up, right, down
+				int Result = -1;
+				for(int Index = 0; Index < 5 && Result == -1; ++Index)
+				{
+					Result = Index;
+					if(!GameServer()->m_World.m_Core.m_Tuning[0].m_PlayerCollision)
+						break;
+					for(int c = 0; c < Num; ++c)
+						if(GameServer()->Collision()->CheckPoint(m_aaSpawnPoints[Type][i] + Positions[Index]) ||
+							distance(aEnts[c]->m_Pos, m_aaSpawnPoints[Type][i] + Positions[Index]) <= aEnts[c]->GetProximityRadius())
+						{
+							Result = -1;
+							break;
+						}
+				}
+				if(Result == -1)
+					continue; // try next spawn point
+
+				P += Positions[Result];
+			}
+
+			float S = EvaluateSpawnPos(pEval, P);
+			if(!pEval->m_Got || (j == 0 && pEval->m_Score > S))
+			{
+				pEval->m_Got = true;
+				pEval->m_Score = S;
+				pEval->m_Pos = P;
+			}
 		}
 	}
 }
@@ -456,7 +466,7 @@ void IGameController::StartRound()
 
 void IGameController::ChangeMap(const char *pToMap)
 {
-	str_copy(g_Config.m_SvMap, pToMap, sizeof(g_Config.m_SvMap));
+	Server()->ChangeMap(pToMap);
 }
 
 void IGameController::OnReset()
@@ -618,6 +628,53 @@ void IGameController::Snap(int SnappingClient)
 		pRaceData->m_Precision = 0;
 		pRaceData->m_RaceFlags = protocol7::RACEFLAG_HIDE_KILLMSG | protocol7::RACEFLAG_KEEP_WANTED_WEAPON;
 	}
+
+	if(GameServer()->Collision()->m_pSwitchers)
+	{
+		int Team = pPlayer && pPlayer->GetCharacter() ? pPlayer->GetCharacter()->Team() : 0;
+
+		if(pPlayer && (pPlayer->GetTeam() == TEAM_SPECTATORS || pPlayer->IsPaused()) && pPlayer->m_SpectatorID != SPEC_FREEVIEW && GameServer()->m_apPlayers[pPlayer->m_SpectatorID] && GameServer()->m_apPlayers[pPlayer->m_SpectatorID]->GetCharacter())
+			Team = GameServer()->m_apPlayers[pPlayer->m_SpectatorID]->GetCharacter()->Team();
+
+		if(Team == TEAM_SUPER)
+			return;
+
+		CNetObj_SwitchState *pSwitchState = static_cast<CNetObj_SwitchState *>(Server()->SnapNewItem(NETOBJTYPE_SWITCHSTATE, Team, sizeof(CNetObj_SwitchState)));
+		if(!pSwitchState)
+			return;
+
+		pSwitchState->m_NumSwitchers = clamp(GameServer()->Collision()->m_NumSwitchers, 0, 255);
+		pSwitchState->m_Status1 = 0;
+		pSwitchState->m_Status2 = 0;
+		pSwitchState->m_Status3 = 0;
+		pSwitchState->m_Status4 = 0;
+		pSwitchState->m_Status5 = 0;
+		pSwitchState->m_Status6 = 0;
+		pSwitchState->m_Status7 = 0;
+		pSwitchState->m_Status8 = 0;
+
+		for(int i = 0; i < pSwitchState->m_NumSwitchers + 1; i++)
+		{
+			int Status = (int)GameServer()->Collision()->m_pSwitchers[i].m_Status[Team];
+
+			if(i < 32)
+				pSwitchState->m_Status1 |= Status << i;
+			else if(i < 64)
+				pSwitchState->m_Status2 |= Status << (i - 32);
+			else if(i < 96)
+				pSwitchState->m_Status3 |= Status << (i - 64);
+			else if(i < 128)
+				pSwitchState->m_Status4 |= Status << (i - 96);
+			else if(i < 160)
+				pSwitchState->m_Status5 |= Status << (i - 128);
+			else if(i < 192)
+				pSwitchState->m_Status6 |= Status << (i - 160);
+			else if(i < 224)
+				pSwitchState->m_Status7 |= Status << (i - 192);
+			else if(i < 256)
+				pSwitchState->m_Status8 |= Status << (i - 224);
+		}
+	}
 }
 
 int IGameController::GetAutoTeam(int NotThisID)
@@ -670,7 +727,7 @@ int IGameController::ClampTeam(int Team)
 	return 0;
 }
 
-int64 IGameController::GetMaskForPlayerWorldEvent(int Asker, int ExceptID)
+int64_t IGameController::GetMaskForPlayerWorldEvent(int Asker, int ExceptID)
 {
 	// Send all world events to everyone by default
 	return CmaskAllExceptOne(ExceptID);
