@@ -27,7 +27,7 @@ public:
 
 	void DoTeamChange(class CPlayer *pPlayer, int Team, bool DoChatMsg = true) override;
 
-	int64 GetMaskForPlayerWorldEvent(int Asker, int ExceptID = -1) override;
+	int64_t GetMaskForPlayerWorldEvent(int Asker, int ExceptID = -1) override;
 
 	void InitTeleporter();
 
