@@ -34,7 +34,7 @@ public:
 	int GetTeam() const { return m_Team; };
 	int GetCID() const { return m_ClientID; };
 	int GetClientVersion() const;
-	bool SetTimerType(int NewType);
+	bool SetTimerType(int TimerType);
 
 	void Tick();
 	void PostTick();
@@ -100,11 +100,6 @@ public:
 	int m_LastActionTick;
 	int m_TeamChangeTick;
 	bool m_SentSemicolonTip;
-	struct
-	{
-		int m_TargetX;
-		int m_TargetY;
-	} m_LatestActivity;
 
 	// network latency calculations
 	struct
@@ -132,8 +127,8 @@ private:
 	int m_Team;
 
 	int m_Paused;
-	int64 m_ForcePauseTime;
-	int64 m_LastPause;
+	int64_t m_ForcePauseTime;
+	int64_t m_LastPause;
 
 	int m_DefEmote;
 	int m_OverrideEmote;
@@ -159,8 +154,8 @@ public:
 	};
 
 	bool m_DND;
-	int64 m_FirstVoteTick;
-	char m_TimeoutCode[64];
+	int64_t m_FirstVoteTick;
+	char m_aTimeoutCode[64];
 
 	void ProcessPause();
 	int Pause(int State, bool Force);
@@ -168,8 +163,8 @@ public:
 	int IsPaused();
 
 	bool IsPlaying();
-	int64 m_Last_KickVote;
-	int64 m_Last_Team;
+	int64_t m_Last_KickVote;
+	int64_t m_Last_Team;
 	int m_ShowOthers;
 	bool m_ShowAll;
 	vec2 m_ShowDistance;
@@ -182,19 +177,23 @@ public:
 
 	bool m_Moderating;
 
-	bool AfkTimer(int new_target_x, int new_target_y); //returns true if kicked
+	bool AfkTimer(CNetObj_PlayerInput *pNewTarget); // returns true if kicked
 	void UpdatePlaytime();
-	void AfkVoteTimer(CNetObj_PlayerInput *NewTarget);
-	int64 m_LastPlaytime;
-	int64 m_LastEyeEmote;
-	int64 m_LastBroadcast;
+	void AfkVoteTimer(CNetObj_PlayerInput *pNewTarget);
+	int64_t m_LastPlaytime;
+	int64_t m_LastEyeEmote;
+	int64_t m_LastBroadcast;
 	bool m_LastBroadcastImportance;
-	int m_LastTarget_x;
-	int m_LastTarget_y;
+
 	CNetObj_PlayerInput *m_pLastTarget;
-	int m_Sent1stAfkWarning; // afk timer's 1st warning after 50% of sv_max_afk_time
-	int m_Sent2ndAfkWarning; // afk timer's 2nd warning after 90% of sv_max_afk_time
-	char m_pAfkMsg[160];
+	bool m_LastTargetInit;
+	/* 
+		afk timer's 1st warning after 50% of sv_max_afk_time
+		2nd warning after 90%
+		kick after reaching 100% of sv_max_afk_time
+	*/
+	bool m_SentAfkWarning[2];
+
 	bool m_EyeEmoteEnabled;
 	int m_TimerType;
 
@@ -203,12 +202,12 @@ public:
 	bool CanOverrideDefaultEmote() const;
 
 	bool m_FirstPacket;
-	int64 m_LastSQLQuery;
+	int64_t m_LastSQLQuery;
 	void ProcessScoreResult(CScorePlayerResult &Result);
 	std::shared_ptr<CScorePlayerResult> m_ScoreQueryResult;
 	std::shared_ptr<CScorePlayerResult> m_ScoreFinishResult;
 	bool m_NotEligibleForFinish;
-	int64 m_EligibleForFinishCheck;
+	int64_t m_EligibleForFinishCheck;
 	bool m_VotedForPractice;
 	int m_SwapTargetsClientID; //Client ID of the swap target for the given player
 };
