@@ -16,7 +16,7 @@ public:
 	CSaveTee();
 	~CSaveTee();
 	void Save(CCharacter *pchr);
-	void Load(CCharacter *pchr, int Team);
+	void Load(CCharacter *pchr, int Team, bool IsSwap = false);
 	char *GetString(const CSaveTeam *pTeam);
 	int FromString(const char *String);
 	void LoadHookedPlayer(const CSaveTeam *pTeam);
@@ -37,6 +37,7 @@ private:
 	int m_NeededFaketuning;
 
 	// Teamstuff
+	int m_TeeStarted;
 	int m_TeeFinished;
 	int m_IsSolo;
 
@@ -57,6 +58,7 @@ private:
 	int m_FreezeTime;
 	int m_FreezeTick;
 	int m_DeepFreeze;
+	int m_LiveFreeze;
 	int m_EndlessHook;
 	int m_DDRaceState;
 
@@ -127,7 +129,7 @@ public:
 	static bool HandleSaveError(int Result, int ClientID, CGameContext *pGameContext);
 
 private:
-	CCharacter *MatchCharacter(int ClientID, int SaveID, bool KeepCurrentWeakStrong);
+	CCharacter *MatchCharacter(int ClientID, int SaveID, bool KeepCurrentCharacter);
 
 	IGameController *m_pController;
 
