@@ -1,13 +1,15 @@
 #ifndef TEST_TEST_H
 #define TEST_TEST_H
+
 class IStorage;
 
 class CTestInfo
 {
 public:
 	CTestInfo();
+	~CTestInfo();
 	IStorage *CreateTestStorage();
-	void DeleteTestStorageFilesOnSuccess();
+	bool m_DeleteTestStorageFilesOnSuccess = false;
 	char m_aFilename[64];
 };
 #endif // TEST_TEST_H
