@@ -73,8 +73,10 @@ void CreateEmptyMap(IStorage *pStorage)
 
 int main(int argc, const char **argv)
 {
+	cmdline_fix(&argc, &argv);
 	dbg_logger_stdout();
 	IStorage *pStorage = CreateStorage("Teeworlds", IStorage::STORAGETYPE_SERVER, argc, argv);
 	CreateEmptyMap(pStorage);
+	cmdline_free(argc, argv);
 	return 0;
 }
