@@ -78,12 +78,13 @@ void GetImageSHA256(uint8_t *pImgBuff, int ImgSize, int Width, int Height, char
 
 int main(int argc, const char **argv)
 {
+	cmdline_fix(&argc, &argv);
 	dbg_logger_stdout();
 
 	IStorage *pStorage = CreateStorage("Teeworlds", IStorage::STORAGETYPE_BASIC, argc, argv);
 	int Index, ID = 0, Type = 0, Size;
 	void *pPtr;
-	char aFileName[1024];
+	char aFileName[IO_MAX_PATH_LENGTH];
 	CDataFileReader DataFile;
 	CDataFileWriter df;
 
@@ -103,8 +104,8 @@ int main(int argc, const char **argv)
 	else
 	{
 		fs_makedir("out");
-		char aBuff[MAX_PATH_LENGTH];
-		pStorage->StripPathAndExtension(argv[1], aBuff, sizeof(aBuff));
+		char aBuff[IO_MAX_PATH_LENGTH];
+		IStorage::StripPathAndExtension(argv[1], aBuff, sizeof(aBuff));
 		str_format(aFileName, sizeof(aFileName), "out/%s.map", aBuff);
 	}
 
@@ -303,7 +304,7 @@ int main(int argc, const char **argv)
 				// Please read the comments inside the functions to understand it
 				GetImageSHA256(pImgBuff, ImgSize, Width, Height, aSHA256Str);
 
-				char aNewName[MAX_PATH_LENGTH];
+				char aNewName[IO_MAX_PATH_LENGTH];
 				int StrLen = str_format(aNewName, sizeof(aNewName) / sizeof(aNewName[0]), "%s_cut_%s", pImgName, aSHA256Str);
 
 				DeletePtr = true;
@@ -324,5 +325,6 @@ int main(int argc, const char **argv)
 	DataFile.Close();
 	df.Finish();
 
+	cmdline_free(argc, argv);
 	return 0;
 }
