@@ -27,35 +27,39 @@ void *g_pNewData = 0;
 
 int LoadPNG(CImageInfo *pImg, const char *pFilename)
 {
-	unsigned char *pBuffer;
 	png_t Png;
 
-	int Error = png_open_file(&Png, pFilename);
+	IOHANDLE File = io_open(pFilename, IOFLAG_READ);
+	if(!File)
+	{
+		dbg_msg("dilate", "failed to open file. filename='%s'", pFilename);
+		return 0;
+	}
+	int Error = png_open_read(&Png, 0, File);
 	if(Error != PNG_NO_ERROR)
 	{
 		dbg_msg("map_replace_image", "failed to open image file. filename='%s', pnglite: %s", pFilename, png_error_string(Error));
-		if(Error != PNG_FILE_ERROR)
-			png_close_file(&Png);
+		io_close(File);
 		return 0;
 	}
 
 	if(Png.depth != 8 || (Png.color_type != PNG_TRUECOLOR && Png.color_type != PNG_TRUECOLOR_ALPHA) || Png.width > (2 << 12) || Png.height > (2 << 12))
 	{
 		dbg_msg("map_replace_image", "invalid image format. filename='%s'", pFilename);
-		png_close_file(&Png);
+		io_close(File);
 		return 0;
 	}
 
-	pBuffer = (unsigned char *)malloc((size_t)Png.width * Png.height * Png.bpp);
+	unsigned char *pBuffer = (unsigned char *)malloc((size_t)Png.width * Png.height * Png.bpp);
 	Error = png_get_data(&Png, pBuffer);
 	if(Error != PNG_NO_ERROR)
 	{
 		dbg_msg("map_convert_07", "failed to read image. filename='%s', pnglite: %s", pFilename, png_error_string(Error));
 		free(pBuffer);
-		png_close_file(&Png);
+		io_close(File);
 		return 0;
 	}
-	png_close_file(&Png);
+	io_close(File);
 
 	pImg->m_Width = Png.width;
 	pImg->m_Height = Png.height;
@@ -106,6 +110,7 @@ void *ReplaceImageItem(void *pItem, int Type, const char *pImgName, const char *
 
 int main(int argc, const char **argv)
 {
+	cmdline_fix(&argc, &argv);
 	dbg_logger_stdout();
 
 	IStorage *pStorage = CreateStorage("Teeworlds", IStorage::STORAGETYPE_BASIC, argc, argv);
@@ -201,5 +206,6 @@ int main(int argc, const char **argv)
 	g_DataWriter.Finish();
 
 	dbg_msg("map_replace_image", "image '%s' replaced", pImageName);
+	cmdline_free(argc, argv);
 	return 0;
 }
