@@ -1,6 +1,7 @@
 #include <engine/shared/uuid_manager.h>
-int main(int argc, char **argv)
+int main(int argc, const char **argv)
 {
+	cmdline_fix(&argc, &argv);
 	dbg_logger_stdout();
 	if(argc != 2)
 	{
@@ -11,4 +12,6 @@ int main(int argc, char **argv)
 	char aBuf[UUID_MAXSTRSIZE];
 	FormatUuid(Uuid, aBuf, sizeof(aBuf));
 	dbg_msg("uuid", "%s", aBuf);
+	cmdline_free(argc, argv);
+	return 0;
 }
